(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     67711,       2215]*)
(*NotebookOutlinePosition[     92555,       2922]*)
(*  CellTagsIndexPosition[     91154,       2865]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["QDENSITY",
    FontSlant->"Italic"],
  "\nShor's factoring algorithms"
}], "Title",
  CellTags->"c:1"],

Cell["Four examples of Shor's factoring algorithm", "Subtitle"],

Cell[TextData[{
  "Last Version: 18-Aug-2005\n",
  Cell[BoxData[
      FormBox[GridBox[{
            {\(\(References\)\(:\)\), \(\([1]\)\ Peter\ W . \ Shor, \ 
              SIAM\ J . \ COMPUT\ 26\ \((5)\)\ 1484\ \((1997)\)\n
              \([2]\)\ Edward\ Gerjuoy\ Am . \ J . \ Phys . \ 
                  73\ \((6)\)\ 521\ \((2005)\)\n
              \([3]\)\ QDENSITY\ Manual, \ Sect . \ 9\)}
            },
          ColumnAlignments->{Right, Left}], TraditionalForm]], 
    "DefinitionBox",
    GridBoxOptions->{RowSpacings->0.4,
    RowLines->False},
    CellTags->"S5.1.1"],
  "\n\nWe present four different implementations of the factoring algorithm. \
The differences between them arise from the method used to perform the \
Quantum Fourier Transform. All of them follow essentially the same steps and \
mostly share the same comments and explanations.\n\n(1) The first case is \
closer to an experiment implementation, using a density matrix, where the QFT \
is explicitly constructed quantum mechanically using Hadamards and Controlled \
gates. \n\n(2) In the second case, a state vector formulation is used , \
instead of a density matrix, to compute the QFT.  This approach saves space, \
as explained in ",
  ButtonBox["QFT.nb",
    ButtonData:>{"QFT.nb", None},
    ButtonStyle->"Hyperlink"],
  ". \n\n(3) The third case, is based on the second approach but also \
explictly constructs the state vectors for both registers, and uses the state \
vector formula for QFT.\n\n(4) Finally, if one is interested in analyzing the \
probabilities for factoring larger numbers, and not in a voracious, closer to \
reality implementation of the QFT, then use this version which replaces QFT \
by  Fourier and InverseFourier, the built-in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions for Discreet Fourier Transforms (DFT), check ",
  ButtonBox["QFT.nb",
    ButtonData:>{"QFT.nb", None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Abstract"],

Cell[TextData[ButtonBox["Click here to get the palette: QDENSpalette.nb",
  ButtonData:>{"QDENSpalette.nb", None},
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[CellGroupData[{

Cell["Shor's Factoring algorithm using density matrix", "Section",
  FontFamily->"Times",
  CellTags->"c:2"],

Cell[BoxData[
    \(Needs["\<QDENSITY`Qdensity`\>"]\)], "Input",
  FontFamily->"Courier New"],

Cell[BoxData[
    \(\(?QDENSITY`Qdensity`*\)\)], "Input",
  FontFamily->"Courier New"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and \
NumberTheory\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.0", "c:3"}],

Cell["\<\
To improve performance we precompile the quantum Fourier transform.  We \
prepare some utilities first, for alter application:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(ControlP[L_, q1_, q2_, k_] := 
        Module[{\[Phi]}, \[Phi] = Exp[2*\ \[Pi]\ *\ \[ImaginaryI]\ /2\^k]; 
          1/4*\((\((3 + \[Phi])\)*TwoOp[L, q1, q2, 0, 0] + \((1 - \[Phi])\)*
                  TwoOp[L, q1, q2, 3, 0] + \((1 - \[Phi])\)*
                  TwoOp[L, q1, q2, 0, 3] + \((\[Phi] - 1)\)*
                  TwoOp[L, q1, q2, 3, 3])\)];\)\)], "Input",
  CellTags->"b:11.0.1"],

Cell[BoxData[
    \(InvertQubits[L_] := \(InvertQubits[L] = 
        Table[If[
            Reverse[IntegerDigits[i, 2, L]] \[Equal] \ 
              IntegerDigits[j, 2, L], 1, 0], {i, 0, 2^L - 1}, {j, 0, 
            2^L - 1}]\)\)], "Input",
  CellTags->"b:11.0.1"],

Cell[BoxData[
    \(\(CompControlP = \ 
        Compile[{{L, _Integer}, {q1, _Integer}, {q2, _Integer}, {k, \
_Integer}}, 
          Block[{Lv = L, qv1 = q1, qv2 = q2, kv = k\ \ }, \ 
            ControlP[Lv, qv1, qv2, 
              kv]], {{\ \ \ ControlP[Lv, qv1, qv2, kv], _Complex, 
              2}}]\ ;\)\)], "Input",
  CellTags->"b:11.0.1"],

Cell[BoxData[
    \(\(CompHad = \ 
        Compile[{{L, _Integer}, {i, _Integer}}, 
          Block[{Lv = L, iv = i}, \ 
            had[Lv, iv]], {{\ \ \ had[Lv, iv], _Real, 2}}]\ \ ;\)\)], "Input",\

  CellTags->"b:11.0.1"],

Cell[BoxData[
    \(\(CompInvertQubits = \ 
        Compile[{{L, _Integer}}, 
          Block[{Lv = L\ }, 
            InvertQubits[Lv]], {{\ \ InvertQubits[Lv], _Real, 
              2}}]\ \ ;\)\)], "Input",
  CellTags->"b:11.0.1"],

Cell[BoxData[
    \(\(\(QuantumFourierTransform = 
        Compile[{{L, _Integer}, {\[CapitalOmega]s, _Complex, 
              2}}, \[IndentingNewLine]Module[{\[CapitalOmega]}, \
\[CapitalOmega] = \[CapitalOmega]s; \[IndentingNewLine]For[i = 1, 
              i \[LessEqual] L, \(i++\), \[CapitalOmega] = 
                N[CompHad[L, i] . \[CapitalOmega]]; 
              For[\ j = i + 1, 
                j \[LessEqual] L, \(j++\), \[CapitalOmega] = 
                  N[CompControlP[L, j, i, 
                        j - i + 
                          1] . \[CapitalOmega]]]]; \[IndentingNewLine]\
\[CapitalOmega] = 
              CompInvertQubits[
                  L] . \[CapitalOmega]; \[CapitalOmega]], \ \
\[IndentingNewLine]{{\ CompHad[L, i], _Real, 2}, {\ 
              CompControlP[L, j, i, j - i + 1], _Complex, 2}, {\ 
              CompInvertQubits[L], _Real, 2}}\ ];\)\(\ \ \)\)\)], "Input",
  CellTags->"b:11.0.1"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(denomi[i_, j_] := 
        Denominator[
          FromContinuedFraction[\ 
            ContinuedFraction[j\/2\^n\_1, i]]];\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell["Flag for the case when factoring an even number ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(\(\ \)\(Even = False;\)\)\)], "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers needed\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.1", "c:4"}],

Cell["M is the number we are trying to factor:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(M = 21;\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(MP = M;\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime, in which case we quit the process:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(While[Mod[MP, 2] \[Equal] 0, Even = True; MP = MP/2];\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[MP \[Equal] 1, Print[M, \ "\< is a power of 2\>"]; 
      Quit[], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[PrimeQ[MP] \[Equal] True, 
      If[Even \[Equal] False, 
        Print[M, "\< is prime, therefore has no factors\>"]; Quit[], 
        Print["\<The prime factors of \>", M, "\< are \>", 2, "\< and \>", 
          MP]; Quit[]], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[TextData[{
  " We need the ",
  StyleBox["minimun number",
    FontWeight->"Bold"],
  " of qubits for both registers. \nHere we should be careful, a high ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " number will lengthen the computation.  \nBy default we give estimates for \
",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  ",based on the fact that ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " should be able to \nrepresent numbers up to M^2, while ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "should allow to write up to M/2. However, in \nmany cases it could be more \
illustrative to lower the value of ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " \n[ ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(Q = 2^Ceiling[Log[2, MP^2]]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[{
    \(\(n\_1 = Ceiling[Log[2, Q]];\)\), "\[IndentingNewLine]", 
    \(\(n\_2 = Ceiling[Log[2, MP]];\)\), "\[IndentingNewLine]", 
    \(n\_1 = 6\), "\[IndentingNewLine]", 
    \(n\_2\)}], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If\ [2^n\_1 < MP^2/4, \ 
      Print[\*"\"\<CHECK YOUR \!\(n\_1\), YOU MAY HAVE PROBLEMS GETTING THE \
PERIOD\>\""], ]\)], "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 2: Load the first register with the integers less than or equal to ",
  Cell[BoxData[
      \(TraditionalForm\`2\^n\_1 - 1\)]],
  "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.2", "c:5"}],

Cell[TextData[{
  " Register one needs to be set to the following: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(|\)\(\[Psi]\_1\)\)\(\[RightAngleBracket]\)\
\) = \ 2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
            j\)\(\ \ \ \)\(\[RightAngleBracket]\)\)\)]],
  " ; which is a \nstate where all ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 3: Select an integer (xguess) coprime to M and load ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(\(xguess\^j\)(
            mod\ M)\)\ into\ the\ second\ register\)]]
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.3", "c:6"}],

Cell[" This is the random Coprime to MP: ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"],

Cell[BoxData[
    \(xguess = 
      Part[Select[Range[MP], GCD[#1, MP] \[Equal] 1 &], 
        Random[Integer, {2, EulerPhi[MP]}]]\)], "Input",
  CellTags->"b:12.0.3"],

Cell[TextData[{
  " The full wave function after we build the second register with  ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "reads:\n ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(\(\(|\)\(\[Psi]\_12\)\)\(\
\[RightAngleBracket]\)\) = \ 
              2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
                j\)\(\ \ \ \)\(\[RightAngleBracket]\)\) | 
          f\_j\)\(\ \ \)\(\[RightAngleBracket]\)\)\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(xguess\^j\)(mod\ M)\)]]
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.4", "c:7"}],

Cell[TextData[{
  "We take an hypothetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "]"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(\(\ \)\(reg2Measurement = 
      Mod[xguess\^Random[Integer, {0, 2\^n\_1 - 1}], MP]\)\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(2\^n\_1\), "\[IndentingNewLine]", 
    \(2\^n\_2\), "\[IndentingNewLine]", 
    \(2\^\((n\_1 + n\_2)\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell[TextData[{
  "The next function gives the number in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  "register in decimal basis from a given number i in the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(n\_1\),
            "TraditionalForm"], "+", \(n\_2\)}], TraditionalForm]]],
  " bases."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(xv[i_] := 
        FromDigits[Take[IntegerDigits[i, 2, n\_1 + n\_2], n\_1], 
          2];\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell[TextData[{
  "We select the positions in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1 + n\_2\)]],
  "register which remain when reg2 has been measured:"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(\(\(dens1 = {};\)\[IndentingNewLine]
    \(Do[If[Mod[xguess^xv[i], MP] \[Equal] reg2Measurement, \ 
          dens1 = Join[\ dens1, {xv[i] + 1\ }], "\<\>"], {i, 0, 
          2\^\((n\_1 + n\_2)\) - 1, 2\^n\_2\ }];\)\)\(\ \)\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(\(reg1 = Table[\ 0, {i, 1, 2^n\_1}];\)\), "\[IndentingNewLine]", 
    \(Do[reg1 = ReplacePart[reg1, 1, dens1[\([i]\)]], {i, 1, 
        Length[dens1]}]\), "\[IndentingNewLine]", 
    \(\(reg1 = reg1/Norm[reg1];\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(densx = Distribute[{dens1, dens1}, List];\)\)], "Input"],

Cell["Share some memory.:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(Share[\ ];\)\)], "Input"],

Cell["Prepere desnity matrix for reg1", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(rho1\  = 
      Compile[{{m, _Integer, 2}, {n, _Integer, 2}}, 
        Module[{t}, t = n; \ ReplacePart[m, 1, densx]]\ \ \ ]\)], "Input"],

Cell[BoxData[
    \(\(\[Rho]1\  = \(1\/\@Length[\ densx]\) 
          rho1[ZeroMatrix[2\^n\_1], densx];\)\)], "Input"],

Cell[BoxData[
    \(\(Share[\ ];\)\)], "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.5", "c:8"}],

Cell["\<\
We apply the inverse of the quantum fourier transform to \[Rho]1:\
\>", "Text",
  FontFamily->"Helvetica"],

Cell[BoxData[
    \(\(rho2\  = 
        Compile[{{x, _Complex, 2}, {y, _Complex, 2}}, \ 
          N[Conjugate[x] . y . x]];\)\)], "Input"],

Cell[BoxData[
    \(\(time1 = SessionTime[];\)\)], "Input",
  CellTags->"b:12.0.5"],

Cell[BoxData[
    \(\(\[Rho]2 = \ 
        rho2\ [QuantumFourierTransform[n\_1, \ 
            N[IdentityMatrix[2\^n\_1]]], \ \[Rho]1\ \ ];\)\)], "Input"],

Cell[BoxData[{
    \(\(time2 = SessionTime[];\)\), "\[IndentingNewLine]", 
    \(Print["\<The QFT took \>", time2 - time1, "\< seconds\>"]\)}], "Input",
  CellTags->"b:12.0.5"],

Cell[BoxData[
    \(Chop[Tr[\[Rho]2]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.6", "c:9"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[
    \(\(MeasProb = {\ };\)\)], "Input"],

Cell[BoxData[
    \(For[num = 0, 
      num < 2\^n\_1, \[CapitalOmega] = Chop[\[Rho]2[\([num + 1, num + 1]\)]]; 
      MeasProb = 
        Append[MeasProb, {num, \[CapitalOmega]}]; \(num++\)]\)], "Input"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[{
    \(\(Plot1 = 
        ListPlot[MeasProb, PlotJoined \[Rule] True, 
          AxesLabel \[Rule] {Measured\ number, Probability}, 
          ImageSize \[Rule] 400];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[MeasProb, PlotJoined \[Rule] True, 
        AxesLabel \[Rule] {Measured\ number, Probability}, 
        PlotRange \[Rule] {0, 0.30}, ImageSize \[Rule] 400];\)\)}], "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.7", "c:10"}],

Cell[TextData[{
  "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
  Cell[BoxData[
      \(TraditionalForm\`xguess\^\(r/2\)\)]],
  "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(\(r = 0;\)\)], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(Alldata = {{Period, Number, "\<f1\>", "\<f2\>", 
            Prob}, {"\<\>", "\<\>", "\<\>", "\<\>", "\<\>"}};\)\), "\
\[IndentingNewLine]", 
    \(\(For[j = 1, 
        j \[LessEqual] Length[MeasProb], \(j++\), \[IndentingNewLine]For[
          i = 2, FromContinuedFraction[\ 
                ContinuedFraction[MeasProb[\([j, 1]\)]\/2\^n\_1, 
                  i - 1]] \[NotEqual] MeasProb[\([j, 1]\)]\/2\^n\_1\  && \ 
            denomi[i, MeasProb[\([j, 1]\)]] <= 
              MP/2, \(i++\), \[IndentingNewLine]r = 
            denomi[i, MeasProb[\([j, 1]\)]]]; \[IndentingNewLine]If[
          IntegerQ[xguess\^\(r/2\)], \[IndentingNewLine]f1 = 
            GCD[xguess\^\(r/2\)\  + 1, MP]\ ; \[IndentingNewLine]f2 = 
            GCD[xguess\^\(r/2\) - 1, MP]; \[IndentingNewLine]If[
            MeasProb[\([j, 2]\)] > 
                0.001 && {f1, f2} \[NotEqual] \ {M, 1} && {f1, 
                  f2} \[NotEqual] {1, M} && f1*f2 \[NotEqual] 1 && 
              f1 \[NotEqual] 1 && 
              f2 \[NotEqual] 1, \[IndentingNewLine]\(Alldata = 
                Join[Alldata, {{r, MeasProb[\([j, 1]\)], f1, f2, 
                      MeasProb[\([j, 
                          2]\)]}}];\)\[IndentingNewLine], "\<you were quite \
unlucky in your xguess!\>"]]];\)\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(Alldata\), "\[IndentingNewLine]", 
    \(\(TotalProbSucc = 
        Sum[Alldata[\([i, 5]\)], {i, 3, 
            Length[Alldata]}];\)\), "\[IndentingNewLine]", 
    \(Print["\<xguess=\>", xguess]\), "\[IndentingNewLine]", 
    \(Print["\<The TOTAL PROBABILITY OF SUCCESS is thus:\>", 
      TotalProbSucc, "\<  with xguess=\>", 
      xguess, \ "\< and measured register=\>", 
      reg2Measurement]\), "\[IndentingNewLine]", 
    \(If\ [TotalProbSucc \[Equal] 0, \ 
      Print["\<Your total probability was zero, you xguess was not very \
lucky!\>"]]\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(SessionTime[]\)], "Input",
  CellTags->"b:12.0.7"],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(a = 
        Table[\ Mod[xguess^i, M], {i, 1, 60}];\)\), "\[IndentingNewLine]", 
    \(ListPlot[a, PlotJoined \[Rule] True]\)}], "Input"],

Cell["Quit", "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm using state vector formula for QFT\
\>", "Section",
  FontFamily->"Times",
  CellTags->"c:11"],

Cell[BoxData[
    \(Needs["\<QDENSITY`Qdensity`\>"]\)], "Input",
  FontFamily->"Courier New"],

Cell[BoxData[
    \(\(?QDENSITY`Qdensity`*\)\)], "Input",
  FontFamily->"Courier New"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and \
NumberTheory\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.0", "c:12"}],

Cell["\<\
To improve performance we precompile the quantum Fourier transform, now using \
state vectors:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[{
    \(Zeroff[vac_, R_] := 
      Sum[vac[\([R[\([i]\)]]\)]/2^i, {i, 1, 
          Length[R]}]\), "\[IndentingNewLine]", 
    \(\(CompZeroff\  = \ 
        Compile[{{vac, _Integer, 1}, {R, _Integer, 1}}, 
          Zeroff[vac, R], {{Zeroff[vac, R], _Real}}];\)\)}], "Input",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(\(QuantumFourierTransformV[
          vac_] := \[IndentingNewLine]Module[{\[CapitalOmega], L}, 
          L = Length[
              vac]; \[CapitalOmega] = \((Ket[0] + 
                Exp[2*\[Pi]*\ \[ImaginaryI]*CompZeroff[vac, Range[L, L]]]*
                  Ket[1])\); \[IndentingNewLine]For[i = 2, 
            i \[LessEqual] L, \(i++\), 
            phass = 2*\[Pi]*\ \[ImaginaryI]*
                Zeroff[vac, 
                  Range[L - i + 1, 
                    L]]; \[CapitalOmega] = \
\[CapitalOmega]\[CircleTimes]\((Ket[0] + 
                    Exp[phass]*
                      Ket[1])\)]; \[IndentingNewLine]\[CapitalOmega] = \
\[CapitalOmega]/2^\((L/2)\); \[CapitalOmega]];\)\(\ \ \)\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(CompQuantumFourierTransformV = 
        Compile[{{v, _Integer, 1}}, \ 
          QuantumFourierTransformV[
            v], {{\ \ \ QuantumFourierTransformV[v], _Complex, 
              2}}]\ \ ;\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(\(IQuantumFourierTransformV[
          vac_] := \[IndentingNewLine]Module[{\[CapitalOmega], L}, 
          L = Length[
              vac]; \[CapitalOmega] = \((Ket[0] + 
                Exp[\(-2\)*\[Pi]*\ \[ImaginaryI]*
                      CompZeroff[vac, Range[L, L]]]*
                  Ket[1])\); \[IndentingNewLine]For[i = 2, 
            i \[LessEqual] L, \(i++\), 
            phass = \(-2\)*\[Pi]*\ \[ImaginaryI]*
                Zeroff[vac, 
                  Range[L - i + 1, 
                    L]]; \[CapitalOmega] = \
\[CapitalOmega]\[CircleTimes]\((Ket[0] + 
                    Exp[phass]*
                      Ket[1])\)]; \[IndentingNewLine]\[CapitalOmega] = \
\[CapitalOmega]/2^\((L/2)\); \[CapitalOmega]];\)\(\ \ \)\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(CompIQuantumFourierTransformV = 
        Compile[{{v, _Integer, 1}}, \ 
          QuantumFourierTransformV[
            v], {{\ \ \ QuantumFourierTransformV[v], _Complex, 
              2}}]\ \ ;\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(denomi[i_, j_] := 
        Denominator[
          FromContinuedFraction[\ 
            ContinuedFraction[j\/2\^n\_1, i]]];\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell["Flag for the case when factoring an even number ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(\(\ \)\(Even = False;\)\)\)], "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.1", "c:13"}],

Cell["M is the number we are trying to factor:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(M = 13*17\)], "Input",
  CellTags->"b:12.0.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(MP = M;\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(While[Mod[MP, 2] \[Equal] 0, Even = True; MP = MP/2];\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[MP \[Equal] 1, Print[M, \ "\< is a power of 2\>"]; 
      Quit[], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[PrimeQ[MP] \[Equal] True, 
      If[Even \[Equal] False, 
        Print[M, "\< is prime, therefore has no factors\>"]; Quit[], 
        Print["\<The prime factors of \>", M, "\< are \>", 2, "\< and \>", 
          MP]; Quit[]], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[TextData[{
  " We need the ",
  StyleBox["minimun number",
    FontWeight->"Bold"],
  " of qubits for both registers. \nHere we should be careful, a high ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " number will lengthen the computation.  \nBy default we give estimates for \
",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  ",based on the fact that ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " should be able to \nrepresent numbers up to M^2, while ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "should allow to write up to M/2. However,  in \nmany cases it could be \
more illustrative to lower the value of ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " \n[ ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(Q = 2^Ceiling[Log[2, MP^2]]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[TextData[{
  "Note the user should experiment with the value of ",
  Cell[BoxData[
      \(TraditionalForm\`n\_\(\(1\)\(\ \ \ \)\)\)]],
  "to reduce the computation time, especially for large M."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[{
    \(\(n\_1 = Ceiling[Log[2, Q]];\)\), "\[IndentingNewLine]", 
    \(\(n\_2 = Ceiling[Log[2, MP]];\)\), "\[IndentingNewLine]", 
    \(n\_1 = 9\), "\[IndentingNewLine]", 
    \(n\_1\), "\[IndentingNewLine]", 
    \(n\_2\)}], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If\ [2^n\_1 < MP^2/4, \ 
      Print[\*"\"\<CHECK YOUR \!\(n\_1\), YOU MAY HAVE PROBLEMS GETTING THE \
PERIOD\>\""], ]\)], "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 2: Load the first register with the integers less than or equal to ",
  Cell[BoxData[
      \(TraditionalForm\`2\^n\_1 - 1\)]],
  "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.2", "c:14"}],

Cell[TextData[{
  " Register one needs to be set to the following: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(|\)\(\[Psi]\_1\)\)\(\[RightAngleBracket]\)\
\) = \ 2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
            j\)\(\ \ \ \)\(\[RightAngleBracket]\)\)\)]],
  " ; which is a \nstate where all ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 3: Select an integer (xguess) coprime to M and load ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(\(xguess\^j\)(
            mod\ M)\)\ into\ the\ second\ register\)]]
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.3", "c:15"}],

Cell[" This is the random Coprime to MP: ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"],

Cell[BoxData[
    \(xguess = 
      Part[Select[Range[MP], GCD[#1, MP] \[Equal] 1 &], 
        Random[Integer, {2, EulerPhi[MP]}]]\)], "Input",
  CellTags->"b:12.0.3"],

Cell[BoxData[
    \(xguess = 16\)], "Input"],

Cell[TextData[{
  " The full wave function after we build the second register with  ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "reads:\n ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(\(\(|\)\(\[Psi]\_12\)\)\(\
\[RightAngleBracket]\)\) = \ 
              2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
                j\)\(\ \ \ \)\(\[RightAngleBracket]\)\) | 
          f\_j\)\(\ \ \)\(\[RightAngleBracket]\)\)\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(xguess\^j\)(mod\ M)\)]]
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.4", "c:16"}],

Cell[TextData[{
  "We take an hypotetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "]"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(\(\ \)\(reg2Measurement = 
      Mod[xguess\^Random[Integer, {0, 2\^n\_1 - 1}], MP]\)\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(2\^n\_1\), "\[IndentingNewLine]", 
    \(2\^n\_2\), "\[IndentingNewLine]", 
    \(2\^\((n\_1 + n\_2)\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell[TextData[{
  "The next function gives the number in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  "register in decimal basis from a given number i in the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(n\_1\),
            "TraditionalForm"], "+", \(n\_2\)}], TraditionalForm]]],
  " bases."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(xv[i_] := 
        FromDigits[Take[IntegerDigits[i, 2, n\_1 + n\_2], n\_1], 
          2];\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell[TextData[{
  "We select the positions in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1 + n\_2\)]],
  "register which remain when reg2 has been measured:"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(\(\(dens1 = {};\)\[IndentingNewLine]
    \(Do[If[Mod[xguess^xv[i], MP] \[Equal] reg2Measurement, \ 
          dens1 = Join[\ dens1, {xv[i] + 1\ }], "\<\>"], {i, 0, 
          2\^\((n\_1 + n\_2)\) - 1, 2\^n\_2\ }];\)\)\(\ \)\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(\(reg1 = Table[\ 0, {i, 1, 2^n\_1}];\)\), "\[IndentingNewLine]", 
    \(Do[reg1 = ReplacePart[reg1, 1, dens1[\([i]\)]], {i, 1, 
        Length[dens1]}]\), "\[IndentingNewLine]", 
    \(\(reg1 = reg1/Norm[reg1];\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell["Share some memory", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(Share[\ ];\)\)], "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.5", "c:17"}],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.5"],

Cell[BoxData[
    \(\(time1 = SessionTime[];\)\)], "Input",
  CellTags->"b:12.0.5"],

Cell[BoxData[{
    \(\(QFTVec = 
        Simplify[
          Sum[reg1[\([i + 1]\)]*
              CompIQuantumFourierTransformV[IntegerDigits[i, 2, n\_1]], {i, 
              0, 2^n\_1 - 1}]];\)\), "\[IndentingNewLine]", 
    \(\(QFTVec = QFTVec;\)\), "\[IndentingNewLine]", 
    \(Print\ ["\<The norm of the inverse fourier transformed vector is: \>", 
      Norm[QFTVec]]\)}], "Input",
  CellTags->"b:12.0.5"],

Cell[BoxData[{
    \(\(time2 = SessionTime[];\)\), "\[IndentingNewLine]", 
    \(Print["\<The QFT took \>", time2 - time1, "\< seconds\>"]\)}], "Input",
  CellTags->"b:12.0.5"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.6", "c:18"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[
    \(\(MeasProb = {\ };\)\)], "Input",
  CellTags->"b:12.0.6"],

Cell[BoxData[
    \(For[num = 0, 
      num < 2\^n\_1, \[CapitalOmega] = 
        Chop[Abs[\(QFTVec[\([num + 1]\)]\)[\([1]\)]]^2]; 
      MeasProb = 
        Append[MeasProb, {num, \[CapitalOmega]}]; \(num++\)]\)], "Input",
  CellTags->"b:12.0.6"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[{
    \(\(Plot1 = 
        ListPlot[MeasProb, PlotJoined \[Rule] True, 
          AxesLabel \[Rule] {Measured\ number, Probability}, 
          ImageSize \[Rule] 400];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[MeasProb, PlotJoined \[Rule] True, 
        AxesLabel \[Rule] {Measured\ number, Probability}, 
        PlotRange \[Rule] {0, 0.30}, ImageSize \[Rule] 400];\)\)}], "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.7", "c:19"}],

Cell[TextData[{
  "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
  Cell[BoxData[
      \(TraditionalForm\`xguess\^\(r/2\)\)]],
  "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(\(r = 0;\)\)], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(Alldata = {{Period, Number, "\<f1\>", "\<f2\>", 
            Prob}, {"\<\>", "\<\>", "\<\>", "\<\>", "\<\>"}};\)\), "\
\[IndentingNewLine]", 
    \(\(For[j = 1, 
        j \[LessEqual] Length[MeasProb], \(j++\), \[IndentingNewLine]For[
          i = 2, FromContinuedFraction[\ 
                ContinuedFraction[MeasProb[\([j, 1]\)]\/2\^n\_1, 
                  i - 1]] \[NotEqual] MeasProb[\([j, 1]\)]\/2\^n\_1\  && \ 
            denomi[i, MeasProb[\([j, 1]\)]] <= 
              MP/2, \(i++\), \[IndentingNewLine]r = 
            denomi[i, MeasProb[\([j, 1]\)]]]; \[IndentingNewLine]If[
          IntegerQ[xguess\^\(r/2\)], \[IndentingNewLine]f1 = 
            GCD[xguess\^\(r/2\)\  + 1, MP]\ ; \[IndentingNewLine]f2 = 
            GCD[xguess\^\(r/2\) - 1, MP]; \[IndentingNewLine]If[
            MeasProb[\([j, 2]\)] > 
                0.001 && {f1, f2} \[NotEqual] \ {M, 1} && {f1, 
                  f2} \[NotEqual] {1, M} && f1*f2 \[NotEqual] 1 && 
              f1 \[NotEqual] 1 && 
              f2 \[NotEqual] 1, \[IndentingNewLine]\(Alldata = 
                Join[Alldata, {{r, MeasProb[\([j, 1]\)], f1, f2, 
                      MeasProb[\([j, 
                          2]\)]}}];\)\[IndentingNewLine], "\<you were quite \
unlucky in your xguess!\>"]]];\)\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(Alldata\), "\[IndentingNewLine]", 
    \(\(TotalProbSucc = 
        Sum[Alldata[\([i, 5]\)], {i, 3, 
            Length[Alldata]}];\)\), "\[IndentingNewLine]", 
    \(Print["\<xguess=\>", xguess]\), "\[IndentingNewLine]", 
    \(Print["\<The TOTAL PROBABILITY OF SUCCESS is thus:\>", 
      TotalProbSucc, "\<  with xguess=\>", 
      xguess, \ "\< and measured register=\>", reg2Measurement]\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(SessionTime[]\)], "Input",
  CellTags->"b:12.0.7"],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(a = 
        Table[\ Mod[xguess^i, M], {i, 1, 60}];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[a, PlotJoined \[Rule] True];\)\)}], "Input"],

Cell["Quit", "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm with explict construction of both registers (using \
state vectors)\
\>", "Section",
  FontFamily->"Times",
  CellTags->"c:20"],

Cell[BoxData[
    \(Needs["\<QDENSITY`Qdensity`\>"]\)], "Input",
  FontFamily->"Courier New"],

Cell[BoxData[
    \(\(?QDENSITY`Qdensity`*\)\)], "Input",
  FontFamily->"Courier New"],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 0: Define functions for QFT and Number Theory[check ",
  ButtonBox["QFT.nb ",
    ButtonData:>{"QFT.nb", None},
    ButtonStyle->"Hyperlink"],
  "for details]"
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.0", "c:21"}],

Cell["\<\
To improve performance we precompile the quantum Fourier transform:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.0"],

Cell[BoxData[{
    \(Zeroff[vac_, R_] := 
      Sum[vac[\([R[\([i]\)]]\)]/2^i, {i, 1, 
          Length[R]}]\), "\[IndentingNewLine]", 
    \(\(CompZeroff\  = \ 
        Compile[{{vac, _Integer, 1}, {R, _Integer, 1}}, 
          Zeroff[vac, R], {{Zeroff[vac, R], _Real}}];\)\)}], "Input",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(\(QuantumFourierTransformV[
          vac_] := \[IndentingNewLine]Module[{\[CapitalOmega], L}, 
          L = Length[
              vac]; \[CapitalOmega] = \((Ket[0] + 
                Exp[2*\[Pi]*\ \[ImaginaryI]*CompZeroff[vac, Range[L, L]]]*
                  Ket[1])\); \[IndentingNewLine]For[i = 2, 
            i \[LessEqual] L, \(i++\), 
            phass = 2*\[Pi]*\ \[ImaginaryI]*
                Zeroff[vac, 
                  Range[L - i + 1, 
                    L]]; \[CapitalOmega] = \
\[CapitalOmega]\[CircleTimes]\((Ket[0] + 
                    Exp[phass]*
                      Ket[1])\)]; \[IndentingNewLine]\[CapitalOmega] = \
\[CapitalOmega]/2^\((L/2)\); \[CapitalOmega]];\)\(\ \ \)\)\)], "Input",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(CompQuantumFourierTransformV = 
        Compile[{{v, _Integer, 1}}, \ 
          QuantumFourierTransformV[
            v], {{\ \ \ QuantumFourierTransformV[v], _Complex, 
              2}}]\ \ ;\)\)], "Input",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(\(IQuantumFourierTransformV[
          vac_] := \[IndentingNewLine]Module[{\[CapitalOmega], L}, 
          L = Length[
              vac]; \[CapitalOmega] = \((Ket[0] + 
                Exp[\(-2\)*\[Pi]*\ \[ImaginaryI]*
                      CompZeroff[vac, Range[L, L]]]*
                  Ket[1])\); \[IndentingNewLine]For[i = 2, 
            i \[LessEqual] L, \(i++\), 
            phass = \(-2\)*\[Pi]*\ \[ImaginaryI]*
                Zeroff[vac, 
                  Range[L - i + 1, 
                    L]]; \[CapitalOmega] = \
\[CapitalOmega]\[CircleTimes]\((Ket[0] + 
                    Exp[phass]*
                      Ket[1])\)]; \[IndentingNewLine]\[CapitalOmega] = \
\[CapitalOmega]/2^\((L/2)\); \[CapitalOmega]];\)\(\ \ \)\)\)], "Input",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(CompIQuantumFourierTransformV = 
        Compile[{{v, _Integer, 1}}, \ 
          QuantumFourierTransformV[
            v], {{\ \ \ QuantumFourierTransformV[v], _Complex, 
              2}}]\ \ ;\)\)], "Input",
  CellTags->"b:13.0.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the continued fraction expansion. 
i is the ith cycle of continued fraction and j is the number measured. \
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(denomi[i_, j_] := 
        Denominator[
          FromContinuedFraction[\ 
            ContinuedFraction[j\/2\^n\_1, i]]];\)\)], "Input",
  CellTags->"b:13.0.0"],

Cell["Flag for the case when factoring an even number ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.0"],

Cell[BoxData[
    \(\(\(\ \)\(Even = False;\)\)\)], "Input",
  CellTags->"b:13.0.0"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39,55,77 and \
determine the computer size required \
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.1", "c:22"}],

Cell["M is the number we are trying to factorize:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(\(M = 35;\)\)], "Input",
  CellTags->"b:13.0.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(\(MP = M;\)\)], "Input",
  CellTags->"b:13.0.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(\(If[Mod[MP, 2] \[Equal] 0, 
        Print["\<Your number is a Even, please revise it\>"]];\)\)], "Input",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(If[PrimeQ[MP] \[Equal] True, 
      If[Even \[Equal] False, 
        Print[M, "\< is prime, therefore has no factors\>"]; Quit[], 
        Print["\<The prime factors of \>", M, "\< are \>", 2, "\< and \>", 
          MP]; Quit[]], ]\)], "Input",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(Q = 2^Ceiling[Log[2, MP^2]]\)], "Input",
  CellTags->"b:13.0.1"],

Cell[TextData[{
  " We need the ",
  StyleBox["minimun number",
    FontWeight->"Bold"],
  " of qubits for both registers. Here we should be careful, \na high ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " number will lengthen the computation, while providing high probability of \
success, \nhowever in many cases it could be more illustrative to lower the \
value of ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " and check the \nresulting probabilities."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.1"],

Cell[BoxData[{
    \(\(n\_1 = Ceiling[Log[2, Q]];\)\), "\[IndentingNewLine]", 
    \(\(n\_2 = Ceiling[Log[2, MP]];\)\), "\[IndentingNewLine]", 
    \(n\_1 = 9\), "\[IndentingNewLine]", 
    \(n\_2\)}], "Input",
  CellTags->"b:13.0.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.1"],

Cell[BoxData[
    \(If\ [2^n\_1 < MP^2/4, \ 
      Print[\*"\"\<CHECK YOUR \!\(n\_1\), YOU MAY HAVE PROBLEMS GETTING THE \
PERIOD\>\""], ]\)], "Input",
  CellTags->"b:13.0.1"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 2: Load the first register with the integers less than or equal to ",
  Cell[BoxData[
      \(TraditionalForm\`2\^n\_1 - 1\)]],
  "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.2", "c:23"}],

Cell[TextData[{
  " Register one needs to be set to the following: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(|\)\(\[Psi]\_1\)\)\(\[RightAngleBracket]\)\
\) = \ 2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
            j\)\(\ \ \ \)\(\[RightAngleBracket]\)\)\)]],
  " ; which is a \nstate where all qubits have been Hadamarded"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.2"],

Cell[BoxData[{
    \(\(\[Psi]\_1 = 
        2^\((\(-n\_1\)/2)\)*Table[1, {2^n\_1}];\)\), "\[IndentingNewLine]", 
    \(Norm[\[Psi]\_1]\)}], "Input",
  CellTags->"b:13.0.2"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 3: Select an integer (xguess) coprime to M and load ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(\(xguess\^j\)(
            mod\ M)\)\ into\ the\ second\ register\)]]
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.3", "c:24"}],

Cell[" This is the random Coprime to MP used in Shor's Algo ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.3"],

Cell[BoxData[
    \(xguess = 
      Part[Select[Range[MP], GCD[#1, MP] \[Equal] 1 &], 
        Random[Integer, {2, EulerPhi[MP]}]]\)], "Input",
  CellTags->"b:13.0.3"],

Cell["\<\
The full state composed of register 1 and register 2 is build in the 
following way:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.3"],

Cell[BoxData[{
    \(\(\[Psi]\_12 = 
        Table[\ 0, {i, 1, 2^\((n\_1 + n\_2)\)}];\)\), "\[IndentingNewLine]", 
    \(Do[\[Psi]\_12 = 
        ReplacePart[\[Psi]\_12, 1, Mod[xguess^i, MP]\  + i*2\^n\_2 + 1], {i, 
        0, 2\^n\_1 - 1}]\), "\[IndentingNewLine]", 
    \(\(\[Psi]\_12 = \[Psi]\_12/Norm[\[Psi]\_12];\)\)}], "Input",
  CellTags->"b:13.0.3"],

Cell[BoxData[{
    \(\(ListPlot[\[Psi]\_12];\)\), "\[IndentingNewLine]", 
    \(\(Norm[\[Psi]\_12];\)\)}], "Input",
  CellTags->"b:13.0.3"],

Cell["Therefore the state of the full system is,", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 4: Measure the second register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.4", "c:25"}],

Cell["\<\
We perform a measurement on register 2, that can be done in the following \
way. First a random 
possible outcome for register two is generated:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.4"],

Cell[BoxData[
    \(\(\(\ \)\(reg2Measurement = 
      Mod[xguess\^Random[Integer, {0, 2\^n\_1 - 1}], MP]\)\)\)], "Input",
  CellTags->"b:13.0.4"],

Cell[TextData[{
  "Then register 1 will be in the following state, making use of the function \
",
  ButtonBox["MeasureKet",
    ButtonData:>{"Tutorial.nb", "MeasureKet"},
    ButtonStyle->"Hyperlink"],
  ": "
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.4"],

Cell[BoxData[{
    \(reg2list = 
      Table[i, {i, n\_1 + 1, n\_1 + n\_2}]\), "\[IndentingNewLine]", 
    \(reg2mes = 
      IntegerDigits[reg2Measurement, 2, n\_2]\), "\[IndentingNewLine]", 
    \(\(\[Psi]\_\(1  P\) = 
        MeasureKet[\[Psi]\_12, reg2list, reg2mes];\)\)}], "Input"],

Cell[BoxData[{
    \(Norm[\[Psi]\_\(1  P\)]\), "\[IndentingNewLine]", 
    \(\(ListPlot[\[Psi]\_\(1  P\)];\)\)}], "Input",
  CellTags->"b:13.0.4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 5: Quantum Fourier Transform register 1", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.5", "c:26"}],

Cell[TextData[{
  "We apply the inverse of the quantum Fourier transform to ",
  Cell[BoxData[
      \(TraditionalForm\`\[Psi]\_\(1  P\)\)]],
  ":"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.5"],

Cell[BoxData[
    \(\(time1 = SessionTime[];\)\)], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[{
    \(\(QFTVec = 
        Simplify[
          Sum[\[Psi]\_\(1  P\)[\([i + 1]\)]*
              CompIQuantumFourierTransformV[IntegerDigits[i, 2, n\_1]], {i, 
              0, 2\^n\_1 - 1}]];\)\), "\[IndentingNewLine]", 
    \(\(QFTVec = QFTVec/Norm[QFTVec];\)\), "\[IndentingNewLine]", 
    \(Norm[QFTVec]\)}], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[
    \(\(time2 = SessionTime[];\)\)], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[
    \(Print["\<The QFT took \>", time2 - time1, "\< Seconds\>"]\)], "Input",
  CellTags->"b:13.0.5"],

Cell["\<\
We store the possible measurements and their probabilities in MeasProb:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.5"],

Cell[BoxData[
    \(\(MeasProb = {\ };\)\)], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[
    \(For[num = 0, 
      num < 2\^n\_1, \[CapitalOmega] = 
        Chop[Abs[\(QFTVec[\([num + 1]\)]\)[\([1]\)]]^2]; 
      MeasProb = 
        Append[MeasProb, {num, \[CapitalOmega]}]; \(num++\)]\)], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[{
    \(\(Normali = 0;\)\), "\[IndentingNewLine]", 
    \(\(Do[
        Normali = Normali + MeasProb[\([i, 2]\)], {i, 1, 
          Length[MeasProb]}];\)\), "\[IndentingNewLine]", 
    \(Normali\)}], "Input",
  CellTags->"b:13.0.5"],

Cell[BoxData[{
    \(\(Plot1 = 
        ListPlot[MeasProb, PlotJoined \[Rule] True, 
          AxesLabel \[Rule] {Meassured\ number, Probability}, 
          ImageSize \[Rule] 400];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[MeasProb, PlotJoined \[Rule] True, 
        AxesLabel \[Rule] {Meassured\ number, Probability}, 
        PlotRange \[Rule] {0, 0.12}, ImageSize \[Rule] 400];\)\)}], "Input",
  CellTags->"b:13.0.5"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead, we study all posible measurements and \
their probabilities\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:13.0.6", "c:27"}],

Cell["\<\
Finally,  we go through all the measurements, compute the periods and check \
the 
factors we would find and their probabilities:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:13.0.6"],

Cell[BoxData[
    \(xguess\)], "Input",
  CellTags->"b:13.0.6"],

Cell[BoxData[
    \(\(r = 0;\)\)], "Input",
  CellTags->"b:13.0.6"],

Cell[BoxData[{
    \(\(Alldata = {{Period, Number, "\<f1\>", "\<f2\>", 
            Prob}, {"\<\>", "\<\>", "\<\>", "\<\>", "\<\>"}};\)\), "\
\[IndentingNewLine]", 
    \(\(For[j = 1, 
        j \[LessEqual] Length[MeasProb], \(j++\), \[IndentingNewLine]For[
          i = 2, FromContinuedFraction[\ 
                ContinuedFraction[MeasProb[\([j, 1]\)]\/2\^n\_1, 
                  i - 1]] \[NotEqual] MeasProb[\([j, 1]\)]\/2\^n\_1\  && \ 
            denomi[i, MeasProb[\([j, 1]\)]] <= 
              MP/2, \(i++\), \[IndentingNewLine]r = 
            denomi[i, MeasProb[\([j, 1]\)]]]; \[IndentingNewLine]If[
          IntegerQ[xguess\^\(r/2\)], \[IndentingNewLine]f1 = 
            GCD[xguess\^\(r/2\)\  + 1, MP]\ ; \[IndentingNewLine]f2 = 
            GCD[xguess\^\(r/2\) - 1, MP]; \[IndentingNewLine]If[
            MeasProb[\([j, 2]\)] > 
                0.001 && {f1, f2} \[NotEqual] \ {M, 1} && {f1, 
                  f2} \[NotEqual] {1, M} && f1*f2 \[NotEqual] 1 && 
              f1 \[NotEqual] 1 && 
              f2 \[NotEqual] 1, \[IndentingNewLine]\(Alldata = 
                Join[Alldata, {{r, MeasProb[\([j, 1]\)], f1, f2, 
                      MeasProb[\([j, 
                          2]\)]}}];\)\[IndentingNewLine], "\<you were quite \
unlucky in your xguess!\>"]]];\)\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(Alldata\), "\[IndentingNewLine]", 
    \(\(TotalProbSucc = 
        Sum[Alldata[\([i, 5]\)], {i, 3, 
            Length[Alldata]}];\)\), "\[IndentingNewLine]", 
    \(Print["\<The TOTAL PROBABILITY OF SUCCESS is thus:\>", 
      TotalProbSucc, "\<  with xguess=\>", 
      xguess, \ "\< and measured register=\>", reg2Measurement]\)}], "Input",
  CellTags->"b:13.0.6"],

Cell[BoxData[
    \(SessionTime[]\)], "Input",
  CellTags->"b:13.0.6"],

Cell[BoxData[
    \(Quit\)], "Input",
  CellTags->"b:13.0.6"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Shor's Factoring algorithm using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " DFT formula"
}], "Section",
  FontFamily->"Times",
  CellTags->"c:28"],

Cell[BoxData[
    \(Needs["\<QDENSITY`Qdensity`\>"]\)], "Input",
  FontFamily->"Courier New"],

Cell[BoxData[
    \(\(?QDENSITY`Qdensity`*\)\)], "Input",
  FontFamily->"Courier New"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and \
NumberTheory\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.0", "c:29"}],

Cell["\<\
In this notebook we make use of the Discrete Fourier Transform already \
present in Mathematica.\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(denomi[i_, j_] := 
        Denominator[
          FromContinuedFraction[\ 
            ContinuedFraction[j\/2\^n\_1, i]]];\)\)], "Input",
  CellTags->"b:12.0.0"],

Cell["Flag for the case when factoring an even number ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.0"],

Cell[BoxData[
    \(\(\(\ \)\(Even = False;\)\)\)], "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.1", "c:30"}],

Cell["M is the number we are trying to factor:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(M = 11*7\)], "Input",
  CellTags->"b:12.0.1"],

Cell["MP is the number we will work with and possibly modify:", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(MP = M;\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(\(While[Mod[MP, 2] \[Equal] 0, Even = True; MP = MP/2];\)\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[MP \[Equal] 1, Print[M, \ "\< is a power of 2\>"]; 
      Quit[], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If[PrimeQ[MP] \[Equal] True, 
      If[Even \[Equal] False, 
        Print[M, "\< is prime, therefore has no factors\>"]; Quit[], 
        Print["\<The prime factors of \>", M, "\< are \>", 2, "\< and \>", 
          MP]; Quit[]], ]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[TextData[{
  " We need the ",
  StyleBox["minimun number",
    FontWeight->"Bold"],
  " of qubits for both registers. \nHere we should be careful, a high ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " number will lengthen the computation.  \nBy default we give estimates for \
",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  ",based on the fact that ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " should be able to \nrepresent numbers up to M^2, while ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "should allow to write up to M/2. However in \nmany cases it could be more \
illustrative to lower the value of ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " \n[ ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(Q = 2^Ceiling[Log[2, MP^2]]\)], "Input",
  CellTags->"b:12.0.1"],

Cell[BoxData[{
    \(n\_1 = Ceiling[Log[2, Q]]\), "\[IndentingNewLine]", 
    \(n\_2 = Ceiling[Log[2, MP]]\), "\[IndentingNewLine]", 
    \(n\_\(\(1\)\(\ \)\)\), "\[IndentingNewLine]", 
    \(n\_2\)}], "Input",
  CellTags->"b:12.0.1"],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.1"],

Cell[BoxData[
    \(If\ [2^n\_1 < MP^2/4, \ 
      Print[\*"\"\<CHECK YOUR \!\(n\_1\), YOU MAY HAVE PROBLEMS GETTING THE \
PERIOD\>\""], ]\)], "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 2: Load the first register with the integers less than or equal to ",
  Cell[BoxData[
      \(TraditionalForm\`2\^n\_1 - 1\)]],
  "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.2", "c:31"}],

Cell[TextData[{
  " Register one needs to be set to the following: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(|\)\(\[Psi]\_1\)\)\(\[RightAngleBracket]\)\
\) = \ 2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
            j\)\(\ \ \ \)\(\[RightAngleBracket]\)\)\)]],
  " ; which is a \nstate where all ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Step 3: Select an integer (xguess) coprime to M and load ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(\(xguess\^j\)(
            mod\ M)\)\ into\ the\ second\ register\)]]
}], "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.3", "c:32"}],

Cell[" This is the random Coprime to MP: ", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"],

Cell[BoxData[
    \(xguess = 
      Part[Select[Range[MP], GCD[#1, MP] \[Equal] 1 &], 
        Random[Integer, {2, EulerPhi[MP]}]]\)], "Input",
  CellTags->"b:12.0.3"],

Cell[TextData[{
  " The full wave function after we build the second register with  ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "reads:\n ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(\(\(\(\(|\)\(\[Psi]\_12\)\)\(\
\[RightAngleBracket]\)\) = \ 
              2\^\(\(-n\_1\)/2\)\ \[Sum]\+\(j = 0\)\%\(2\^n\_1 - 1\)\  | 
                j\)\(\ \ \ \)\(\[RightAngleBracket]\)\) | 
          f\_j\)\(\ \ \)\(\[RightAngleBracket]\)\)\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j \[Congruent] \ \(xguess\^j\)(mod\ M)\)]]
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.4", "c:33"}],

Cell[TextData[{
  "We take an hypotetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
  Cell[BoxData[
      \(TraditionalForm\`f\_j\)]],
  "]"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(\(\(\ \)\(reg2Measurement = 
      Mod[xguess\^Random[Integer, {0, 2\^n\_1 - 1}], MP]\)\)\)], "Input",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(2\^n\_1\), "\[IndentingNewLine]", 
    \(2\^n\_2\), "\[IndentingNewLine]", 
    \(2\^\((n\_1 + n\_2)\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell[TextData[{
  "The next function gives the number in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  "register in decimal basis from a given number i in the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(n\_1\),
            "TraditionalForm"], "+", \(n\_2\)}], TraditionalForm]]],
  " bases."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(\(xv[i_] := 
        FromDigits[Take[IntegerDigits[i, 2, n\_1 + n\_2], n\_1], 
          2];\)\), "\[IndentingNewLine]", 
    \(Compxv = 
      Compile[{{i, _Integer, 0}}, xv[i], {{xv[i], _Integer, 0}}]\)}], "Input",\

  CellTags->"b:12.0.4"],

Cell[TextData[{
  "We select the positions in the ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1 + n\_2\)]],
  "register which remain when reg2 has been measured:"
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(\(dens1 = {};\)\), "\[IndentingNewLine]", 
    \( (*\(Do[
          If[Mod[xguess^xv[i], MP] \[Equal] reg2Measurement, \ 
            dens1 = Join[\ dens1, {xv[i] + 1\ }], "\<\>"], {i, 0, 
            2\^\((n\_1 + n\_2)\) - 1, 2\^n\_2\ }];\)\ *) \(Do[
        xx = Compxv[i*2\^n\_2]; 
        If[Mod[xguess^xx, MP] \[Equal] reg2Measurement, \ 
          dens1 = Join[\ dens1, {xx + 1\ }], "\<\>"], {i, 0, 
          2\^n\_1 - 1\ }];\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.4"],

Cell[BoxData[{
    \(\(reg1 = Table[\ 0, {i, 1, 2^n\_1}];\)\), "\[IndentingNewLine]", 
    \(Do[reg1 = ReplacePart[reg1, 1, dens1[\([i]\)]], {i, 1, 
        Length[dens1]}]\), "\[IndentingNewLine]", 
    \(\(reg1 = reg1/Norm[reg1];\)\)}], "Input",
  CellTags->"b:12.0.4"],

Cell[BoxData[
    \(Share[\ ]\)], "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.5", "c:34"}],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.5"],

Cell[BoxData[
    \(\(time1 = SessionTime[];\)\)], "Input",
  CellTags->"b:12.0.5"],

Cell[BoxData[
    \(\(QFTVec = Fourier[reg1];\)\)], "Input"],

Cell[BoxData[{
    \(\(time2 = SessionTime[];\)\), "\[IndentingNewLine]", 
    \(Print["\<The QFT took \>", time2 - time1, "\< seconds\>"]\)}], "Input",
  CellTags->"b:12.0.5"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.6", "c:35"}],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[
    \(\(MeasProb = {\ };\)\)], "Input",
  CellTags->"b:12.0.6"],

Cell[BoxData[
    \(For[num = 0, 
      num < 2\^n\_1, \[CapitalOmega] = Chop[Abs[QFTVec[\([num + 1]\)]]^2]; 
      MeasProb = 
        Append[MeasProb, {num, \[CapitalOmega]}]; \(num++\)]\)], "Input",
  CellTags->"b:12.0.6"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.6"],

Cell[BoxData[{
    \(\(Plot1 = 
        ListPlot[MeasProb, PlotJoined \[Rule] True, 
          AxesLabel \[Rule] {Measured\ number, Probability}, 
          ImageSize \[Rule] 400];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[MeasProb, PlotJoined \[Rule] True, 
        AxesLabel \[Rule] {Measured\ number, Probability}, 
        PlotRange \[Rule] {0, 0.30}, ImageSize \[Rule] 400];\)\)}], "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
  CellFrame->True,
  FontFamily->"Times",
  CellTags->{"b:12.0.7", "c:36"}],

Cell[TextData[{
  "Finally,  we go through all the measurements, compute the periods and \
check the \nfactors we would find and their probabilities. \nSchematically: \
we search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
  Cell[BoxData[
      \(TraditionalForm\`xguess\^\(r/2\)\)]],
  "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(\(r = 0;\)\)], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(Alldata = {{Period, Number, "\<f1\>", "\<f2\>", 
            Prob}, {"\<\>", "\<\>", "\<\>", "\<\>", "\<\>"}};\)\), "\
\[IndentingNewLine]", 
    \(\(For[j = 1, 
        j \[LessEqual] Length[MeasProb], \(j++\), \[IndentingNewLine]For[
          i = 2, FromContinuedFraction[\ 
                ContinuedFraction[MeasProb[\([j, 1]\)]\/2\^n\_1, 
                  i - 1]] \[NotEqual] MeasProb[\([j, 1]\)]\/2\^n\_1\  && \ 
            denomi[i, MeasProb[\([j, 1]\)]] <= 
              MP/2, \(i++\), \[IndentingNewLine]r = 
            denomi[i, MeasProb[\([j, 1]\)]]]; \[IndentingNewLine]If[
          IntegerQ[xguess\^\(r/2\)], \[IndentingNewLine]f1 = 
            GCD[xguess\^\(r/2\)\  + 1, MP]\ ; \[IndentingNewLine]f2 = 
            GCD[xguess\^\(r/2\) - 1, MP]; \[IndentingNewLine]If[
            MeasProb[\([j, 2]\)] > 
                0.001 && {f1, f2} \[NotEqual] \ {M, 1} && {f1, 
                  f2} \[NotEqual] {1, M} && f1*f2 \[NotEqual] 1 && 
              f1 \[NotEqual] 1 && 
              f2 \[NotEqual] 1, \[IndentingNewLine]\(Alldata = 
                Join[Alldata, {{r, MeasProb[\([j, 1]\)], f1, f2, 
                      MeasProb[\([j, 
                          2]\)]}}];\)\[IndentingNewLine], "\<you were quite \
unlucky in your xguess!\>"]]];\)\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(Alldata\), "\[IndentingNewLine]", 
    \(\(TotalProbSucc = 
        Sum[Alldata[\([i, 5]\)], {i, 3, 
            Length[Alldata]}];\)\), "\[IndentingNewLine]", 
    \(Print["\<xguess=\>", xguess]\), "\[IndentingNewLine]", 
    \(Print["\<The TOTAL PROBABILITY OF SUCCESS is thus:\>", 
      TotalProbSucc, "\<  with xguess=\>", 
      xguess, \ "\< and measured register=\>", reg2Measurement]\)}], "Input",
  CellTags->"b:12.0.7"],

Cell[BoxData[
    \(SessionTime[]\)], "Input",
  CellTags->"b:12.0.7"],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
  FontFamily->"Helvetica",
  CellTags->"b:12.0.7"],

Cell[BoxData[{
    \(\(a = 
        Table[\ Mod[xguess^i, M], {i, 1, 60}];\)\), "\[IndentingNewLine]", 
    \(ListPlot[a, PlotJoined \[Rule] True]\)}], "Input"],

Cell["Quit", "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]]
},
FrontEndVersion->"5.2 for X",
ScreenRectangle->{{0, 1280}, {0, 800}},
ShowPageBreaks->False,
WindowToolbars->{},
WindowSize->{929, 746},
WindowMargins->{{10, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingStartingPageNumber->53,
PrintingPageRange->{1, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "export", "home", \
"brunojd", "PITT", "QC", "QdensityPK"}, "35_22.ps", CharacterEncoding -> \
"iso8859-1"],
"Magnification"->1},
ShowSelection->True,
TaggingRules:>{"PaginationCache" -> {{2005, 8, 9, 3, 36, \
22.593709`8.106562514035977}, {53, 53, 53, 53, 53, 53, 53, 53, 54, 54, 54, \
54, 54, 54, 54, 54, 54, 54, 54, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, \
55, 55, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 56, 57, 57, 57, 57, 57, \
57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 58, 58, 58, 58, 58, 58, 58, \
58, 58, 58, 58, 58, 59, 59, 59, 59, 59, 59, 59, 59, 60, 60, 60, 60, 60, 60, \
60, 60, 60, 60, 60, 60, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, \
61, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 63, 63, 63, 63, 63, 63, \
63, 63, 63, 63, 63, 63, 63, 63, 64, 64, 64, 64, 64, 64, 64, 64, 64, 65, 65, \
65, 65, 65, 65, 65, 65, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 67, \
67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 68, 68, 68, 68, 68, 68, 68, \
68, 68, 68, 68, 68, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, \
70, 70, 70, 70, 70, 70, 70, 70, 71, 71, 71, 71, 71, 71, 71, 71, 71, 71, 71, \
71, 71, 71, 71, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 73, 73, 73, \
73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 74, 74, 74, 74, 74, 74, 74, 74, 74, \
74, 74, 74, 74, 75, 75, 75, 75, 75, 75, 75, 76, 76}, Automatic}},
CellLabelAutoDelete->True,
Magnification->1,
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:1"->{
    Cell[1754, 51, 130, 5, 122, "Title",
      CellTags->"c:1"]},
  "c:2"->{
    Cell[4100, 108, 108, 2, 62, "Section",
      CellTags->"c:2"]},
  "b:12.0.0"->{
    Cell[4418, 122, 185, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:3"}],
    Cell[4606, 130, 201, 5, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[7240, 200, 255, 7, 58, "Text",
      CellTags->"b:12.0.0"],
    Cell[7498, 209, 183, 5, 64, "Input",
      CellTags->"b:12.0.0"],
    Cell[7684, 216, 115, 2, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[7802, 220, 84, 2, 47, "Input",
      CellTags->"b:12.0.0"],
    Cell[21845, 696, 186, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:12"}],
    Cell[22034, 704, 169, 5, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[22206, 711, 312, 7, 63, "Input",
      CellTags->"b:12.0.0"],
    Cell[22521, 720, 763, 16, 95, "Input",
      CellTags->"b:12.0.0"],
    Cell[23287, 738, 255, 6, 63, "Input",
      CellTags->"b:12.0.0"],
    Cell[23545, 746, 797, 17, 95, "Input",
      CellTags->"b:12.0.0"],
    Cell[24345, 765, 256, 6, 63, "Input",
      CellTags->"b:12.0.0"],
    Cell[24604, 773, 255, 7, 58, "Text",
      CellTags->"b:12.0.0"],
    Cell[24862, 782, 183, 5, 64, "Input",
      CellTags->"b:12.0.0"],
    Cell[25048, 789, 115, 2, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[25166, 793, 84, 2, 47, "Input",
      CellTags->"b:12.0.0"],
    Cell[53788, 1747, 186, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:29"}],
    Cell[53977, 1755, 171, 5, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[54151, 1762, 255, 7, 58, "Text",
      CellTags->"b:12.0.0"],
    Cell[54409, 1771, 183, 5, 64, "Input",
      CellTags->"b:12.0.0"],
    Cell[54595, 1778, 115, 2, 28, "Text",
      CellTags->"b:12.0.0"],
    Cell[54713, 1782, 84, 2, 47, "Input",
      CellTags->"b:12.0.0"]},
  "c:3"->{
    Cell[4418, 122, 185, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:3"}]},
  "b:11.0.1"->{
    Cell[4810, 137, 412, 7, 103, "Input",
      CellTags->"b:11.0.1"],
    Cell[5225, 146, 265, 6, 79, "Input",
      CellTags->"b:11.0.1"],
    Cell[5493, 154, 347, 8, 79, "Input",
      CellTags->"b:11.0.1"],
    Cell[5843, 164, 225, 6, 47, "Input",
      CellTags->"b:11.0.1"],
    Cell[6071, 172, 232, 6, 47, "Input",
      CellTags->"b:11.0.1"],
    Cell[6306, 180, 931, 18, 111, "Input",
      CellTags->"b:11.0.1"]},
  "b:12.0.1"->{
    Cell[7923, 227, 216, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:4"}],
    Cell[8142, 235, 107, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[8252, 239, 68, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[8323, 243, 122, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[8448, 247, 68, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[8519, 251, 195, 5, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[8717, 258, 114, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[8834, 262, 127, 3, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[8964, 267, 289, 6, 63, "Input",
      CellTags->"b:12.0.1"],
    Cell[9256, 275, 985, 31, 178, "Text",
      CellTags->"b:12.0.1"],
    Cell[10244, 308, 84, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[10331, 312, 234, 5, 95, "Input",
      CellTags->"b:12.0.1"],
    Cell[10568, 319, 252, 7, 58, "Text",
      CellTags->"b:12.0.1"],
    Cell[10823, 328, 175, 4, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[25287, 800, 210, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:13"}],
    Cell[25500, 808, 107, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[25610, 812, 66, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[25679, 816, 122, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[25804, 820, 68, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[25875, 824, 178, 5, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[26056, 831, 114, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[26173, 835, 127, 3, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[26303, 840, 289, 6, 63, "Input",
      CellTags->"b:12.0.1"],
    Cell[26595, 848, 986, 31, 178, "Text",
      CellTags->"b:12.0.1"],
    Cell[27584, 881, 84, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[27671, 885, 263, 7, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[27937, 894, 272, 6, 111, "Input",
      CellTags->"b:12.0.1"],
    Cell[28212, 902, 252, 7, 58, "Text",
      CellTags->"b:12.0.1"],
    Cell[28467, 911, 175, 4, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[54834, 1789, 210, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:30"}],
    Cell[55047, 1797, 107, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[55157, 1801, 65, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[55225, 1805, 122, 2, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[55350, 1809, 68, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[55421, 1813, 160, 5, 28, "Text",
      CellTags->"b:12.0.1"],
    Cell[55584, 1820, 114, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[55701, 1824, 127, 3, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[55831, 1829, 289, 6, 63, "Input",
      CellTags->"b:12.0.1"],
    Cell[56123, 1837, 984, 31, 178, "Text",
      CellTags->"b:12.0.1"],
    Cell[57110, 1870, 84, 2, 47, "Input",
      CellTags->"b:12.0.1"],
    Cell[57197, 1874, 234, 5, 95, "Input",
      CellTags->"b:12.0.1"],
    Cell[57434, 1881, 251, 7, 58, "Text",
      CellTags->"b:12.0.1"],
    Cell[57688, 1890, 175, 4, 47, "Input",
      CellTags->"b:12.0.1"]},
  "c:4"->{
    Cell[7923, 227, 216, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:4"}]},
  "b:12.0.2"->{
    Cell[11035, 337, 291, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:5"}],
    Cell[11329, 347, 542, 13, 95, "Text",
      CellTags->"b:12.0.2"],
    Cell[28679, 920, 292, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:14"}],
    Cell[28974, 930, 542, 13, 95, "Text",
      CellTags->"b:12.0.2"],
    Cell[57900, 1899, 292, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:31"}],
    Cell[58195, 1909, 542, 13, 95, "Text",
      CellTags->"b:12.0.2"]},
  "c:5"->{
    Cell[11035, 337, 291, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:5"}]},
  "b:12.0.3"->{
    Cell[11908, 365, 304, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:6"}],
    Cell[12215, 375, 102, 2, 28, "Text",
      CellTags->"b:12.0.3"],
    Cell[12320, 379, 167, 4, 47, "Input",
      CellTags->"b:12.0.3"],
    Cell[12490, 385, 612, 16, 62, "Text",
      CellTags->"b:12.0.3"],
    Cell[29553, 948, 305, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:15"}],
    Cell[29861, 958, 102, 2, 28, "Text",
      CellTags->"b:12.0.3"],
    Cell[29966, 962, 167, 4, 47, "Input",
      CellTags->"b:12.0.3"],
    Cell[30183, 971, 612, 16, 62, "Text",
      CellTags->"b:12.0.3"],
    Cell[58774, 1927, 305, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:32"}],
    Cell[59082, 1937, 102, 2, 28, "Text",
      CellTags->"b:12.0.3"],
    Cell[59187, 1941, 167, 4, 47, "Input",
      CellTags->"b:12.0.3"],
    Cell[59357, 1947, 612, 16, 62, "Text",
      CellTags->"b:12.0.3"]},
  "c:6"->{
    Cell[11908, 365, 304, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:6"}]},
  "b:12.0.4"->{
    Cell[13139, 406, 152, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:7"}],
    Cell[13294, 411, 270, 8, 58, "Text",
      CellTags->"b:12.0.4"],
    Cell[13567, 421, 146, 3, 53, "Input",
      CellTags->"b:12.0.4"],
    Cell[13716, 426, 161, 4, 81, "Input",
      CellTags->"b:12.0.4"],
    Cell[13880, 432, 395, 13, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[14278, 447, 151, 4, 47, "Input",
      CellTags->"b:12.0.4"],
    Cell[14432, 453, 228, 7, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[14663, 462, 284, 5, 67, "Input",
      CellTags->"b:12.0.4"],
    Cell[14950, 469, 137, 4, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[15090, 475, 271, 5, 79, "Input",
      CellTags->"b:12.0.4"],
    Cell[15445, 485, 86, 2, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[15584, 492, 98, 2, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[15966, 505, 71, 2, 47, "Input",
      CellTags->"b:12.0.4"],
    Cell[30832, 992, 153, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:16"}],
    Cell[30988, 997, 269, 8, 58, "Text",
      CellTags->"b:12.0.4"],
    Cell[31260, 1007, 146, 3, 53, "Input",
      CellTags->"b:12.0.4"],
    Cell[31409, 1012, 161, 4, 81, "Input",
      CellTags->"b:12.0.4"],
    Cell[31573, 1018, 395, 13, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[31971, 1033, 151, 4, 47, "Input",
      CellTags->"b:12.0.4"],
    Cell[32125, 1039, 228, 7, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[32356, 1048, 284, 5, 67, "Input",
      CellTags->"b:12.0.4"],
    Cell[32643, 1055, 137, 4, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[32783, 1061, 271, 5, 79, "Input",
      CellTags->"b:12.0.4"],
    Cell[33057, 1068, 84, 2, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[33144, 1072, 71, 2, 47, "Input",
      CellTags->"b:12.0.4"],
    Cell[60006, 1968, 153, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:33"}],
    Cell[60162, 1973, 269, 8, 58, "Text",
      CellTags->"b:12.0.4"],
    Cell[60434, 1983, 146, 3, 53, "Input",
      CellTags->"b:12.0.4"],
    Cell[60583, 1988, 161, 4, 81, "Input",
      CellTags->"b:12.0.4"],
    Cell[60747, 1994, 395, 13, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[61145, 2009, 263, 7, 63, "Input",
      CellTags->"b:12.0.4"],
    Cell[61411, 2018, 228, 7, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[61642, 2027, 493, 10, 80, "Input",
      CellTags->"b:12.0.4"],
    Cell[62138, 2039, 137, 4, 28, "Text",
      CellTags->"b:12.0.4"],
    Cell[62278, 2045, 271, 5, 79, "Input",
      CellTags->"b:12.0.4"],
    Cell[62552, 2052, 66, 2, 47, "Input",
      CellTags->"b:12.0.4"]},
  "c:7"->{
    Cell[13139, 406, 152, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:7"}]},
  "b:12.0.5"->{
    Cell[16074, 512, 146, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:8"}],
    Cell[16484, 527, 83, 2, 47, "Input",
      CellTags->"b:12.0.5"],
    Cell[16727, 536, 176, 3, 63, "Input",
      CellTags->"b:12.0.5"],
    Cell[33252, 1079, 147, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:17"}],
    Cell[33402, 1084, 163, 5, 28, "Text",
      CellTags->"b:12.0.5"],
    Cell[33568, 1091, 83, 2, 47, "Input",
      CellTags->"b:12.0.5"],
    Cell[33654, 1095, 411, 9, 79, "Input",
      CellTags->"b:12.0.5"],
    Cell[34068, 1106, 176, 3, 63, "Input",
      CellTags->"b:12.0.5"],
    Cell[62655, 2059, 147, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:34"}],
    Cell[62805, 2064, 163, 5, 28, "Text",
      CellTags->"b:12.0.5"],
    Cell[62971, 2071, 83, 2, 47, "Input",
      CellTags->"b:12.0.5"],
    Cell[63120, 2078, 176, 3, 63, "Input",
      CellTags->"b:12.0.5"]},
  "c:8"->{
    Cell[16074, 512, 146, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:8"}]},
  "b:12.0.6"->{
    Cell[16993, 547, 193, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:9"}],
    Cell[17189, 555, 157, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[17612, 571, 159, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[17774, 578, 423, 8, 79, "Input",
      CellTags->"b:12.0.6"],
    Cell[34281, 1114, 194, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:18"}],
    Cell[34478, 1122, 157, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[34638, 1129, 77, 2, 47, "Input",
      CellTags->"b:12.0.6"],
    Cell[34718, 1133, 247, 6, 63, "Input",
      CellTags->"b:12.0.6"],
    Cell[34968, 1141, 159, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[35130, 1148, 423, 8, 79, "Input",
      CellTags->"b:12.0.6"],
    Cell[63333, 2086, 194, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:35"}],
    Cell[63530, 2094, 157, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[63690, 2101, 77, 2, 47, "Input",
      CellTags->"b:12.0.6"],
    Cell[63770, 2105, 225, 5, 47, "Input",
      CellTags->"b:12.0.6"],
    Cell[63998, 2112, 159, 5, 28, "Text",
      CellTags->"b:12.0.6"],
    Cell[64160, 2119, 423, 8, 79, "Input",
      CellTags->"b:12.0.6"]},
  "c:9"->{
    Cell[16993, 547, 193, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:9"}]},
  "b:12.0.7"->{
    Cell[18234, 591, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:10"}],
    Cell[18410, 598, 474, 10, 148, "Text",
      CellTags->"b:12.0.7"],
    Cell[18887, 610, 67, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[18957, 614, 1330, 24, 250, "Input",
      CellTags->"b:12.0.7"],
    Cell[20290, 640, 611, 13, 127, "Input",
      CellTags->"b:12.0.7"],
    Cell[20904, 655, 70, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[20977, 659, 267, 7, 88, "Text",
      CellTags->"b:12.0.7"],
    Cell[21410, 673, 45, 1, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[35590, 1161, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:19"}],
    Cell[35766, 1168, 474, 10, 148, "Text",
      CellTags->"b:12.0.7"],
    Cell[36243, 1180, 67, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[36313, 1184, 1330, 24, 250, "Input",
      CellTags->"b:12.0.7"],
    Cell[37646, 1210, 451, 9, 111, "Input",
      CellTags->"b:12.0.7"],
    Cell[38100, 1221, 70, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[38173, 1225, 267, 7, 88, "Text",
      CellTags->"b:12.0.7"],
    Cell[38611, 1239, 45, 1, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[51489, 1680, 1330, 24, 250, "Input",
      CellTags->"b:12.0.7"],
    Cell[64620, 2132, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:36"}],
    Cell[64796, 2139, 476, 10, 148, "Text",
      CellTags->"b:12.0.7"],
    Cell[65275, 2151, 67, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[65345, 2155, 1330, 24, 250, "Input",
      CellTags->"b:12.0.7"],
    Cell[66678, 2181, 451, 9, 111, "Input",
      CellTags->"b:12.0.7"],
    Cell[67132, 2192, 70, 2, 47, "Input",
      CellTags->"b:12.0.7"],
    Cell[67205, 2196, 267, 7, 88, "Text",
      CellTags->"b:12.0.7"],
    Cell[67638, 2210, 45, 1, 47, "Input",
      CellTags->"b:12.0.7"]},
  "c:10"->{
    Cell[18234, 591, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:10"}]},
  "c:11"->{
    Cell[21504, 680, 131, 4, 42, "Section",
      CellTags->"c:11"]},
  "c:12"->{
    Cell[21845, 696, 186, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:12"}]},
  "c:13"->{
    Cell[25287, 800, 210, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:13"}]},
  "c:14"->{
    Cell[28679, 920, 292, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:14"}]},
  "c:15"->{
    Cell[29553, 948, 305, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:15"}]},
  "c:16"->{
    Cell[30832, 992, 153, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:16"}]},
  "c:17"->{
    Cell[33252, 1079, 147, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:17"}]},
  "c:18"->{
    Cell[34281, 1114, 194, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:18"}]},
  "c:19"->{
    Cell[35590, 1161, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:19"}]},
  "c:20"->{
    Cell[38705, 1246, 164, 5, 42, "Section",
      CellTags->"c:20"]},
  "b:13.0.0"->{
    Cell[39079, 1263, 277, 9, 69, "Subsection",
      CellTags->{"b:13.0.0", "c:21"}],
    Cell[39359, 1274, 142, 4, 28, "Text",
      CellTags->"b:13.0.0"],
    Cell[39504, 1280, 312, 7, 63, "Input",
      CellTags->"b:13.0.0"],
    Cell[39819, 1289, 763, 16, 95, "Input",
      CellTags->"b:13.0.0"],
    Cell[40585, 1307, 255, 6, 63, "Input",
      CellTags->"b:13.0.0"],
    Cell[40843, 1315, 797, 17, 95, "Input",
      CellTags->"b:13.0.0"],
    Cell[41643, 1334, 256, 6, 63, "Input",
      CellTags->"b:13.0.0"],
    Cell[41902, 1342, 263, 6, 58, "Text",
      CellTags->"b:13.0.0"],
    Cell[42168, 1350, 183, 5, 64, "Input",
      CellTags->"b:13.0.0"],
    Cell[42354, 1357, 115, 2, 28, "Text",
      CellTags->"b:13.0.0"],
    Cell[42472, 1361, 84, 2, 47, "Input",
      CellTags->"b:13.0.0"]},
  "c:21"->{
    Cell[39079, 1263, 277, 9, 69, "Subsection",
      CellTags->{"b:13.0.0", "c:21"}]},
  "b:13.0.1"->{
    Cell[42593, 1368, 218, 6, 53, "Subsection",
      CellTags->{"b:13.0.1", "c:22"}],
    Cell[42814, 1376, 110, 2, 28, "Text",
      CellTags->"b:13.0.1"],
    Cell[42927, 1380, 68, 2, 47, "Input",
      CellTags->"b:13.0.1"],
    Cell[42998, 1384, 122, 2, 28, "Text",
      CellTags->"b:13.0.1"],
    Cell[43123, 1388, 68, 2, 47, "Input",
      CellTags->"b:13.0.1"],
    Cell[43194, 1392, 178, 5, 28, "Text",
      CellTags->"b:13.0.1"],
    Cell[43375, 1399, 150, 3, 47, "Input",
      CellTags->"b:13.0.1"],
    Cell[43528, 1404, 289, 6, 63, "Input",
      CellTags->"b:13.0.1"],
    Cell[43820, 1412, 84, 2, 47, "Input",
      CellTags->"b:13.0.1"],
    Cell[43907, 1416, 542, 15, 118, "Text",
      CellTags->"b:13.0.1"],
    Cell[44452, 1433, 234, 5, 95, "Input",
      CellTags->"b:13.0.1"],
    Cell[44689, 1440, 251, 7, 58, "Text",
      CellTags->"b:13.0.1"],
    Cell[44943, 1449, 175, 4, 47, "Input",
      CellTags->"b:13.0.1"]},
  "c:22"->{
    Cell[42593, 1368, 218, 6, 53, "Subsection",
      CellTags->{"b:13.0.1", "c:22"}]},
  "b:13.0.2"->{
    Cell[45155, 1458, 292, 8, 53, "Subsection",
      CellTags->{"b:13.0.2", "c:23"}],
    Cell[45450, 1468, 415, 9, 65, "Text",
      CellTags->"b:13.0.2"],
    Cell[45868, 1479, 172, 4, 63, "Input",
      CellTags->"b:13.0.2"]},
  "c:23"->{
    Cell[45155, 1458, 292, 8, 53, "Subsection",
      CellTags->{"b:13.0.2", "c:23"}]},
  "b:13.0.3"->{
    Cell[46077, 1488, 305, 8, 53, "Subsection",
      CellTags->{"b:13.0.3", "c:24"}],
    Cell[46385, 1498, 121, 2, 28, "Text",
      CellTags->"b:13.0.3"],
    Cell[46509, 1502, 167, 4, 47, "Input",
      CellTags->"b:13.0.3"],
    Cell[46679, 1508, 159, 5, 58, "Text",
      CellTags->"b:13.0.3"],
    Cell[46841, 1515, 357, 7, 79, "Input",
      CellTags->"b:13.0.3"],
    Cell[47201, 1524, 139, 3, 63, "Input",
      CellTags->"b:13.0.3"],
    Cell[47343, 1529, 109, 2, 28, "Text",
      CellTags->"b:13.0.3"]},
  "c:24"->{
    Cell[46077, 1488, 305, 8, 53, "Subsection",
      CellTags->{"b:13.0.3", "c:24"}]},
  "b:13.0.4"->{
    Cell[47489, 1536, 133, 3, 53, "Subsection",
      CellTags->{"b:13.0.4", "c:25"}],
    Cell[47625, 1541, 219, 6, 58, "Text",
      CellTags->"b:13.0.4"],
    Cell[47847, 1549, 146, 3, 53, "Input",
      CellTags->"b:13.0.4"],
    Cell[47996, 1554, 272, 9, 28, "Text",
      CellTags->"b:13.0.4"],
    Cell[48561, 1573, 146, 3, 63, "Input",
      CellTags->"b:13.0.4"]},
  "c:25"->{
    Cell[47489, 1536, 133, 3, 53, "Subsection",
      CellTags->{"b:13.0.4", "c:25"}]},
  "b:13.0.5"->{
    Cell[48744, 1581, 142, 3, 53, "Subsection",
      CellTags->{"b:13.0.5", "c:26"}],
    Cell[48889, 1586, 210, 7, 28, "Text",
      CellTags->"b:13.0.5"],
    Cell[49102, 1595, 83, 2, 47, "Input",
      CellTags->"b:13.0.5"],
    Cell[49188, 1599, 358, 8, 79, "Input",
      CellTags->"b:13.0.5"],
    Cell[49549, 1609, 83, 2, 47, "Input",
      CellTags->"b:13.0.5"],
    Cell[49635, 1613, 114, 2, 47, "Input",
      CellTags->"b:13.0.5"],
    Cell[49752, 1617, 146, 4, 28, "Text",
      CellTags->"b:13.0.5"],
    Cell[49901, 1623, 77, 2, 47, "Input",
      CellTags->"b:13.0.5"],
    Cell[49981, 1627, 247, 6, 63, "Input",
      CellTags->"b:13.0.5"],
    Cell[50231, 1635, 245, 6, 79, "Input",
      CellTags->"b:13.0.5"],
    Cell[50479, 1643, 425, 8, 79, "Input",
      CellTags->"b:13.0.5"]},
  "c:26"->{
    Cell[48744, 1581, 142, 3, 53, "Subsection",
      CellTags->{"b:13.0.5", "c:26"}]},
  "b:13.0.6"->{
    Cell[50941, 1656, 202, 6, 53, "Subsection",
      CellTags->{"b:13.0.6", "c:27"}],
    Cell[51146, 1664, 204, 6, 58, "Text",
      CellTags->"b:13.0.6"],
    Cell[51353, 1672, 63, 2, 47, "Input",
      CellTags->"b:13.0.6"],
    Cell[51419, 1676, 67, 2, 47, "Input",
      CellTags->"b:13.0.6"],
    Cell[52822, 1706, 389, 8, 95, "Input",
      CellTags->"b:13.0.6"],
    Cell[53214, 1716, 70, 2, 47, "Input",
      CellTags->"b:13.0.6"],
    Cell[53287, 1720, 61, 2, 47, "Input",
      CellTags->"b:13.0.6"]},
  "c:27"->{
    Cell[50941, 1656, 202, 6, 53, "Subsection",
      CellTags->{"b:13.0.6", "c:27"}]},
  "c:28"->{
    Cell[53397, 1728, 181, 7, 42, "Section",
      CellTags->"c:28"]},
  "c:29"->{
    Cell[53788, 1747, 186, 6, 69, "Subsection",
      CellTags->{"b:12.0.0", "c:29"}]},
  "c:30"->{
    Cell[54834, 1789, 210, 6, 53, "Subsection",
      CellTags->{"b:12.0.1", "c:30"}]},
  "c:31"->{
    Cell[57900, 1899, 292, 8, 53, "Subsection",
      CellTags->{"b:12.0.2", "c:31"}]},
  "c:32"->{
    Cell[58774, 1927, 305, 8, 53, "Subsection",
      CellTags->{"b:12.0.3", "c:32"}]},
  "c:33"->{
    Cell[60006, 1968, 153, 3, 53, "Subsection",
      CellTags->{"b:12.0.4", "c:33"}]},
  "c:34"->{
    Cell[62655, 2059, 147, 3, 53, "Subsection",
      CellTags->{"b:12.0.5", "c:34"}]},
  "c:35"->{
    Cell[63333, 2086, 194, 6, 53, "Subsection",
      CellTags->{"b:12.0.6", "c:35"}]},
  "c:36"->{
    Cell[64620, 2132, 173, 5, 53, "Subsection",
      CellTags->{"b:12.0.7", "c:36"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:1", 70134, 2265},
  {"c:2", 70211, 2268},
  {"b:12.0.0", 70295, 2271},
  {"c:3", 71974, 2318},
  {"b:11.0.1", 72075, 2321},
  {"b:12.0.1", 72515, 2334},
  {"c:4", 75610, 2421},
  {"b:12.0.2", 75711, 2424},
  {"c:5", 76195, 2437},
  {"b:12.0.3", 76297, 2440},
  {"c:6", 77206, 2465},
  {"b:12.0.4", 77308, 2468},
  {"c:7", 79924, 2541},
  {"b:12.0.5", 80026, 2544},
  {"c:8", 80938, 2569},
  {"b:12.0.6", 81040, 2572},
  {"c:9", 82236, 2605},
  {"b:12.0.7", 82338, 2608},
  {"c:10", 84195, 2659},
  {"c:11", 84294, 2662},
  {"c:12", 84376, 2665},
  {"c:13", 84475, 2668},
  {"c:14", 84574, 2671},
  {"c:15", 84673, 2674},
  {"c:16", 84772, 2677},
  {"c:17", 84871, 2680},
  {"c:18", 84971, 2683},
  {"c:19", 85071, 2686},
  {"c:20", 85171, 2689},
  {"b:13.0.0", 85258, 2692},
  {"c:21", 86076, 2715},
  {"b:13.0.1", 86180, 2718},
  {"c:22", 87138, 2745},
  {"b:13.0.2", 87242, 2748},
  {"c:23", 87485, 2755},
  {"b:13.0.3", 87589, 2758},
  {"c:24", 88118, 2773},
  {"b:13.0.4", 88222, 2776},
  {"c:25", 88608, 2787},
  {"b:13.0.5", 88712, 2790},
  {"c:26", 89527, 2813},
  {"b:13.0.6", 89631, 2816},
  {"c:27", 90158, 2831},
  {"c:28", 90258, 2834},
  {"c:29", 90341, 2837},
  {"c:30", 90441, 2840},
  {"c:31", 90541, 2843},
  {"c:32", 90641, 2846},
  {"c:33", 90741, 2849},
  {"c:34", 90841, 2852},
  {"c:35", 90941, 2855},
  {"c:36", 91041, 2858}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 130, 5, 122, "Title",
  CellTags->"c:1"],
Cell[1887, 58, 63, 0, 41, "Subtitle"],
Cell[1953, 60, 1967, 40, 389, "Abstract"],
Cell[3923, 102, 152, 2, 28, "Text"],

Cell[CellGroupData[{
Cell[4100, 108, 108, 2, 62, "Section",
  CellTags->"c:2"],
Cell[4211, 112, 93, 2, 47, "Input"],
Cell[4307, 116, 86, 2, 47, "Input"],

Cell[CellGroupData[{
Cell[4418, 122, 185, 6, 69, "Subsection",
  CellTags->{"b:12.0.0", "c:3"}],
Cell[4606, 130, 201, 5, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[4810, 137, 412, 7, 103, "Input",
  CellTags->"b:11.0.1"],
Cell[5225, 146, 265, 6, 79, "Input",
  CellTags->"b:11.0.1"],
Cell[5493, 154, 347, 8, 79, "Input",
  CellTags->"b:11.0.1"],
Cell[5843, 164, 225, 6, 47, "Input",
  CellTags->"b:11.0.1"],
Cell[6071, 172, 232, 6, 47, "Input",
  CellTags->"b:11.0.1"],
Cell[6306, 180, 931, 18, 111, "Input",
  CellTags->"b:11.0.1"],
Cell[7240, 200, 255, 7, 58, "Text",
  CellTags->"b:12.0.0"],
Cell[7498, 209, 183, 5, 64, "Input",
  CellTags->"b:12.0.0"],
Cell[7684, 216, 115, 2, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[7802, 220, 84, 2, 47, "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{
Cell[7923, 227, 216, 6, 53, "Subsection",
  CellTags->{"b:12.0.1", "c:4"}],
Cell[8142, 235, 107, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[8252, 239, 68, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[8323, 243, 122, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[8448, 247, 68, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[8519, 251, 195, 5, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[8717, 258, 114, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[8834, 262, 127, 3, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[8964, 267, 289, 6, 63, "Input",
  CellTags->"b:12.0.1"],
Cell[9256, 275, 985, 31, 178, "Text",
  CellTags->"b:12.0.1"],
Cell[10244, 308, 84, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[10331, 312, 234, 5, 95, "Input",
  CellTags->"b:12.0.1"],
Cell[10568, 319, 252, 7, 58, "Text",
  CellTags->"b:12.0.1"],
Cell[10823, 328, 175, 4, 47, "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[11035, 337, 291, 8, 53, "Subsection",
  CellTags->{"b:12.0.2", "c:5"}],
Cell[11329, 347, 542, 13, 95, "Text",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[11908, 365, 304, 8, 53, "Subsection",
  CellTags->{"b:12.0.3", "c:6"}],
Cell[12215, 375, 102, 2, 28, "Text",
  CellTags->"b:12.0.3"],
Cell[12320, 379, 167, 4, 47, "Input",
  CellTags->"b:12.0.3"],
Cell[12490, 385, 612, 16, 62, "Text",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{
Cell[13139, 406, 152, 3, 53, "Subsection",
  CellTags->{"b:12.0.4", "c:7"}],
Cell[13294, 411, 270, 8, 58, "Text",
  CellTags->"b:12.0.4"],
Cell[13567, 421, 146, 3, 53, "Input",
  CellTags->"b:12.0.4"],
Cell[13716, 426, 161, 4, 81, "Input",
  CellTags->"b:12.0.4"],
Cell[13880, 432, 395, 13, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[14278, 447, 151, 4, 47, "Input",
  CellTags->"b:12.0.4"],
Cell[14432, 453, 228, 7, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[14663, 462, 284, 5, 67, "Input",
  CellTags->"b:12.0.4"],
Cell[14950, 469, 137, 4, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[15090, 475, 271, 5, 79, "Input",
  CellTags->"b:12.0.4"],
Cell[15364, 482, 78, 1, 47, "Input"],
Cell[15445, 485, 86, 2, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[15534, 489, 47, 1, 47, "Input"],
Cell[15584, 492, 98, 2, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[15685, 496, 157, 3, 47, "Input"],
Cell[15845, 501, 118, 2, 71, "Input"],
Cell[15966, 505, 71, 2, 47, "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{
Cell[16074, 512, 146, 3, 53, "Subsection",
  CellTags->{"b:12.0.5", "c:8"}],
Cell[16223, 517, 116, 3, 28, "Text"],
Cell[16342, 522, 139, 3, 47, "Input"],
Cell[16484, 527, 83, 2, 47, "Input",
  CellTags->"b:12.0.5"],
Cell[16570, 531, 154, 3, 47, "Input"],
Cell[16727, 536, 176, 3, 63, "Input",
  CellTags->"b:12.0.5"],
Cell[16906, 541, 50, 1, 47, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[16993, 547, 193, 6, 53, "Subsection",
  CellTags->{"b:12.0.6", "c:9"}],
Cell[17189, 555, 157, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[17349, 562, 53, 1, 47, "Input"],
Cell[17405, 565, 204, 4, 47, "Input"],
Cell[17612, 571, 159, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[17774, 578, 423, 8, 79, "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{
Cell[18234, 591, 173, 5, 53, "Subsection",
  CellTags->{"b:12.0.7", "c:10"}],
Cell[18410, 598, 474, 10, 148, "Text",
  CellTags->"b:12.0.7"],
Cell[18887, 610, 67, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[18957, 614, 1330, 24, 250, "Input",
  CellTags->"b:12.0.7"],
Cell[20290, 640, 611, 13, 127, "Input",
  CellTags->"b:12.0.7"],
Cell[20904, 655, 70, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[20977, 659, 267, 7, 88, "Text",
  CellTags->"b:12.0.7"],
Cell[21247, 668, 160, 3, 63, "Input"],
Cell[21410, 673, 45, 1, 47, "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[21504, 680, 131, 4, 42, "Section",
  CellTags->"c:11"],
Cell[21638, 686, 93, 2, 47, "Input"],
Cell[21734, 690, 86, 2, 47, "Input"],

Cell[CellGroupData[{
Cell[21845, 696, 186, 6, 69, "Subsection",
  CellTags->{"b:12.0.0", "c:12"}],
Cell[22034, 704, 169, 5, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[22206, 711, 312, 7, 63, "Input",
  CellTags->"b:12.0.0"],
Cell[22521, 720, 763, 16, 95, "Input",
  CellTags->"b:12.0.0"],
Cell[23287, 738, 255, 6, 63, "Input",
  CellTags->"b:12.0.0"],
Cell[23545, 746, 797, 17, 95, "Input",
  CellTags->"b:12.0.0"],
Cell[24345, 765, 256, 6, 63, "Input",
  CellTags->"b:12.0.0"],
Cell[24604, 773, 255, 7, 58, "Text",
  CellTags->"b:12.0.0"],
Cell[24862, 782, 183, 5, 64, "Input",
  CellTags->"b:12.0.0"],
Cell[25048, 789, 115, 2, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[25166, 793, 84, 2, 47, "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{
Cell[25287, 800, 210, 6, 53, "Subsection",
  CellTags->{"b:12.0.1", "c:13"}],
Cell[25500, 808, 107, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[25610, 812, 66, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[25679, 816, 122, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[25804, 820, 68, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[25875, 824, 178, 5, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[26056, 831, 114, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[26173, 835, 127, 3, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[26303, 840, 289, 6, 63, "Input",
  CellTags->"b:12.0.1"],
Cell[26595, 848, 986, 31, 178, "Text",
  CellTags->"b:12.0.1"],
Cell[27584, 881, 84, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[27671, 885, 263, 7, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[27937, 894, 272, 6, 111, "Input",
  CellTags->"b:12.0.1"],
Cell[28212, 902, 252, 7, 58, "Text",
  CellTags->"b:12.0.1"],
Cell[28467, 911, 175, 4, 47, "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[28679, 920, 292, 8, 53, "Subsection",
  CellTags->{"b:12.0.2", "c:14"}],
Cell[28974, 930, 542, 13, 95, "Text",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[29553, 948, 305, 8, 53, "Subsection",
  CellTags->{"b:12.0.3", "c:15"}],
Cell[29861, 958, 102, 2, 28, "Text",
  CellTags->"b:12.0.3"],
Cell[29966, 962, 167, 4, 47, "Input",
  CellTags->"b:12.0.3"],
Cell[30136, 968, 44, 1, 47, "Input"],
Cell[30183, 971, 612, 16, 62, "Text",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{
Cell[30832, 992, 153, 3, 53, "Subsection",
  CellTags->{"b:12.0.4", "c:16"}],
Cell[30988, 997, 269, 8, 58, "Text",
  CellTags->"b:12.0.4"],
Cell[31260, 1007, 146, 3, 53, "Input",
  CellTags->"b:12.0.4"],
Cell[31409, 1012, 161, 4, 81, "Input",
  CellTags->"b:12.0.4"],
Cell[31573, 1018, 395, 13, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[31971, 1033, 151, 4, 47, "Input",
  CellTags->"b:12.0.4"],
Cell[32125, 1039, 228, 7, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[32356, 1048, 284, 5, 67, "Input",
  CellTags->"b:12.0.4"],
Cell[32643, 1055, 137, 4, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[32783, 1061, 271, 5, 79, "Input",
  CellTags->"b:12.0.4"],
Cell[33057, 1068, 84, 2, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[33144, 1072, 71, 2, 47, "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{
Cell[33252, 1079, 147, 3, 53, "Subsection",
  CellTags->{"b:12.0.5", "c:17"}],
Cell[33402, 1084, 163, 5, 28, "Text",
  CellTags->"b:12.0.5"],
Cell[33568, 1091, 83, 2, 47, "Input",
  CellTags->"b:12.0.5"],
Cell[33654, 1095, 411, 9, 79, "Input",
  CellTags->"b:12.0.5"],
Cell[34068, 1106, 176, 3, 63, "Input",
  CellTags->"b:12.0.5"]
}, Closed]],

Cell[CellGroupData[{
Cell[34281, 1114, 194, 6, 53, "Subsection",
  CellTags->{"b:12.0.6", "c:18"}],
Cell[34478, 1122, 157, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[34638, 1129, 77, 2, 47, "Input",
  CellTags->"b:12.0.6"],
Cell[34718, 1133, 247, 6, 63, "Input",
  CellTags->"b:12.0.6"],
Cell[34968, 1141, 159, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[35130, 1148, 423, 8, 79, "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{
Cell[35590, 1161, 173, 5, 53, "Subsection",
  CellTags->{"b:12.0.7", "c:19"}],
Cell[35766, 1168, 474, 10, 148, "Text",
  CellTags->"b:12.0.7"],
Cell[36243, 1180, 67, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[36313, 1184, 1330, 24, 250, "Input",
  CellTags->"b:12.0.7"],
Cell[37646, 1210, 451, 9, 111, "Input",
  CellTags->"b:12.0.7"],
Cell[38100, 1221, 70, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[38173, 1225, 267, 7, 88, "Text",
  CellTags->"b:12.0.7"],
Cell[38443, 1234, 165, 3, 63, "Input"],
Cell[38611, 1239, 45, 1, 47, "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[38705, 1246, 164, 5, 42, "Section",
  CellTags->"c:20"],
Cell[38872, 1253, 93, 2, 47, "Input"],
Cell[38968, 1257, 86, 2, 47, "Input"],

Cell[CellGroupData[{
Cell[39079, 1263, 277, 9, 69, "Subsection",
  CellTags->{"b:13.0.0", "c:21"}],
Cell[39359, 1274, 142, 4, 28, "Text",
  CellTags->"b:13.0.0"],
Cell[39504, 1280, 312, 7, 63, "Input",
  CellTags->"b:13.0.0"],
Cell[39819, 1289, 763, 16, 95, "Input",
  CellTags->"b:13.0.0"],
Cell[40585, 1307, 255, 6, 63, "Input",
  CellTags->"b:13.0.0"],
Cell[40843, 1315, 797, 17, 95, "Input",
  CellTags->"b:13.0.0"],
Cell[41643, 1334, 256, 6, 63, "Input",
  CellTags->"b:13.0.0"],
Cell[41902, 1342, 263, 6, 58, "Text",
  CellTags->"b:13.0.0"],
Cell[42168, 1350, 183, 5, 64, "Input",
  CellTags->"b:13.0.0"],
Cell[42354, 1357, 115, 2, 28, "Text",
  CellTags->"b:13.0.0"],
Cell[42472, 1361, 84, 2, 47, "Input",
  CellTags->"b:13.0.0"]
}, Closed]],

Cell[CellGroupData[{
Cell[42593, 1368, 218, 6, 53, "Subsection",
  CellTags->{"b:13.0.1", "c:22"}],
Cell[42814, 1376, 110, 2, 28, "Text",
  CellTags->"b:13.0.1"],
Cell[42927, 1380, 68, 2, 47, "Input",
  CellTags->"b:13.0.1"],
Cell[42998, 1384, 122, 2, 28, "Text",
  CellTags->"b:13.0.1"],
Cell[43123, 1388, 68, 2, 47, "Input",
  CellTags->"b:13.0.1"],
Cell[43194, 1392, 178, 5, 28, "Text",
  CellTags->"b:13.0.1"],
Cell[43375, 1399, 150, 3, 47, "Input",
  CellTags->"b:13.0.1"],
Cell[43528, 1404, 289, 6, 63, "Input",
  CellTags->"b:13.0.1"],
Cell[43820, 1412, 84, 2, 47, "Input",
  CellTags->"b:13.0.1"],
Cell[43907, 1416, 542, 15, 118, "Text",
  CellTags->"b:13.0.1"],
Cell[44452, 1433, 234, 5, 95, "Input",
  CellTags->"b:13.0.1"],
Cell[44689, 1440, 251, 7, 58, "Text",
  CellTags->"b:13.0.1"],
Cell[44943, 1449, 175, 4, 47, "Input",
  CellTags->"b:13.0.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[45155, 1458, 292, 8, 53, "Subsection",
  CellTags->{"b:13.0.2", "c:23"}],
Cell[45450, 1468, 415, 9, 65, "Text",
  CellTags->"b:13.0.2"],
Cell[45868, 1479, 172, 4, 63, "Input",
  CellTags->"b:13.0.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[46077, 1488, 305, 8, 53, "Subsection",
  CellTags->{"b:13.0.3", "c:24"}],
Cell[46385, 1498, 121, 2, 28, "Text",
  CellTags->"b:13.0.3"],
Cell[46509, 1502, 167, 4, 47, "Input",
  CellTags->"b:13.0.3"],
Cell[46679, 1508, 159, 5, 58, "Text",
  CellTags->"b:13.0.3"],
Cell[46841, 1515, 357, 7, 79, "Input",
  CellTags->"b:13.0.3"],
Cell[47201, 1524, 139, 3, 63, "Input",
  CellTags->"b:13.0.3"],
Cell[47343, 1529, 109, 2, 28, "Text",
  CellTags->"b:13.0.3"]
}, Closed]],

Cell[CellGroupData[{
Cell[47489, 1536, 133, 3, 53, "Subsection",
  CellTags->{"b:13.0.4", "c:25"}],
Cell[47625, 1541, 219, 6, 58, "Text",
  CellTags->"b:13.0.4"],
Cell[47847, 1549, 146, 3, 53, "Input",
  CellTags->"b:13.0.4"],
Cell[47996, 1554, 272, 9, 28, "Text",
  CellTags->"b:13.0.4"],
Cell[48271, 1565, 287, 6, 79, "Input"],
Cell[48561, 1573, 146, 3, 63, "Input",
  CellTags->"b:13.0.4"]
}, Closed]],

Cell[CellGroupData[{
Cell[48744, 1581, 142, 3, 53, "Subsection",
  CellTags->{"b:13.0.5", "c:26"}],
Cell[48889, 1586, 210, 7, 28, "Text",
  CellTags->"b:13.0.5"],
Cell[49102, 1595, 83, 2, 47, "Input",
  CellTags->"b:13.0.5"],
Cell[49188, 1599, 358, 8, 79, "Input",
  CellTags->"b:13.0.5"],
Cell[49549, 1609, 83, 2, 47, "Input",
  CellTags->"b:13.0.5"],
Cell[49635, 1613, 114, 2, 47, "Input",
  CellTags->"b:13.0.5"],
Cell[49752, 1617, 146, 4, 28, "Text",
  CellTags->"b:13.0.5"],
Cell[49901, 1623, 77, 2, 47, "Input",
  CellTags->"b:13.0.5"],
Cell[49981, 1627, 247, 6, 63, "Input",
  CellTags->"b:13.0.5"],
Cell[50231, 1635, 245, 6, 79, "Input",
  CellTags->"b:13.0.5"],
Cell[50479, 1643, 425, 8, 79, "Input",
  CellTags->"b:13.0.5"]
}, Closed]],

Cell[CellGroupData[{
Cell[50941, 1656, 202, 6, 53, "Subsection",
  CellTags->{"b:13.0.6", "c:27"}],
Cell[51146, 1664, 204, 6, 58, "Text",
  CellTags->"b:13.0.6"],
Cell[51353, 1672, 63, 2, 47, "Input",
  CellTags->"b:13.0.6"],
Cell[51419, 1676, 67, 2, 47, "Input",
  CellTags->"b:13.0.6"],
Cell[51489, 1680, 1330, 24, 250, "Input",
  CellTags->"b:12.0.7"],
Cell[52822, 1706, 389, 8, 95, "Input",
  CellTags->"b:13.0.6"],
Cell[53214, 1716, 70, 2, 47, "Input",
  CellTags->"b:13.0.6"],
Cell[53287, 1720, 61, 2, 47, "Input",
  CellTags->"b:13.0.6"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[53397, 1728, 181, 7, 42, "Section",
  CellTags->"c:28"],
Cell[53581, 1737, 93, 2, 47, "Input"],
Cell[53677, 1741, 86, 2, 47, "Input"],

Cell[CellGroupData[{
Cell[53788, 1747, 186, 6, 69, "Subsection",
  CellTags->{"b:12.0.0", "c:29"}],
Cell[53977, 1755, 171, 5, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[54151, 1762, 255, 7, 58, "Text",
  CellTags->"b:12.0.0"],
Cell[54409, 1771, 183, 5, 64, "Input",
  CellTags->"b:12.0.0"],
Cell[54595, 1778, 115, 2, 28, "Text",
  CellTags->"b:12.0.0"],
Cell[54713, 1782, 84, 2, 47, "Input",
  CellTags->"b:12.0.0"]
}, Closed]],

Cell[CellGroupData[{
Cell[54834, 1789, 210, 6, 53, "Subsection",
  CellTags->{"b:12.0.1", "c:30"}],
Cell[55047, 1797, 107, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[55157, 1801, 65, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[55225, 1805, 122, 2, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[55350, 1809, 68, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[55421, 1813, 160, 5, 28, "Text",
  CellTags->"b:12.0.1"],
Cell[55584, 1820, 114, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[55701, 1824, 127, 3, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[55831, 1829, 289, 6, 63, "Input",
  CellTags->"b:12.0.1"],
Cell[56123, 1837, 984, 31, 178, "Text",
  CellTags->"b:12.0.1"],
Cell[57110, 1870, 84, 2, 47, "Input",
  CellTags->"b:12.0.1"],
Cell[57197, 1874, 234, 5, 95, "Input",
  CellTags->"b:12.0.1"],
Cell[57434, 1881, 251, 7, 58, "Text",
  CellTags->"b:12.0.1"],
Cell[57688, 1890, 175, 4, 47, "Input",
  CellTags->"b:12.0.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[57900, 1899, 292, 8, 53, "Subsection",
  CellTags->{"b:12.0.2", "c:31"}],
Cell[58195, 1909, 542, 13, 95, "Text",
  CellTags->"b:12.0.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[58774, 1927, 305, 8, 53, "Subsection",
  CellTags->{"b:12.0.3", "c:32"}],
Cell[59082, 1937, 102, 2, 28, "Text",
  CellTags->"b:12.0.3"],
Cell[59187, 1941, 167, 4, 47, "Input",
  CellTags->"b:12.0.3"],
Cell[59357, 1947, 612, 16, 62, "Text",
  CellTags->"b:12.0.3"]
}, Closed]],

Cell[CellGroupData[{
Cell[60006, 1968, 153, 3, 53, "Subsection",
  CellTags->{"b:12.0.4", "c:33"}],
Cell[60162, 1973, 269, 8, 58, "Text",
  CellTags->"b:12.0.4"],
Cell[60434, 1983, 146, 3, 53, "Input",
  CellTags->"b:12.0.4"],
Cell[60583, 1988, 161, 4, 81, "Input",
  CellTags->"b:12.0.4"],
Cell[60747, 1994, 395, 13, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[61145, 2009, 263, 7, 63, "Input",
  CellTags->"b:12.0.4"],
Cell[61411, 2018, 228, 7, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[61642, 2027, 493, 10, 80, "Input",
  CellTags->"b:12.0.4"],
Cell[62138, 2039, 137, 4, 28, "Text",
  CellTags->"b:12.0.4"],
Cell[62278, 2045, 271, 5, 79, "Input",
  CellTags->"b:12.0.4"],
Cell[62552, 2052, 66, 2, 47, "Input",
  CellTags->"b:12.0.4"]
}, Closed]],

Cell[CellGroupData[{
Cell[62655, 2059, 147, 3, 53, "Subsection",
  CellTags->{"b:12.0.5", "c:34"}],
Cell[62805, 2064, 163, 5, 28, "Text",
  CellTags->"b:12.0.5"],
Cell[62971, 2071, 83, 2, 47, "Input",
  CellTags->"b:12.0.5"],
Cell[63057, 2075, 60, 1, 47, "Input"],
Cell[63120, 2078, 176, 3, 63, "Input",
  CellTags->"b:12.0.5"]
}, Closed]],

Cell[CellGroupData[{
Cell[63333, 2086, 194, 6, 53, "Subsection",
  CellTags->{"b:12.0.6", "c:35"}],
Cell[63530, 2094, 157, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[63690, 2101, 77, 2, 47, "Input",
  CellTags->"b:12.0.6"],
Cell[63770, 2105, 225, 5, 47, "Input",
  CellTags->"b:12.0.6"],
Cell[63998, 2112, 159, 5, 28, "Text",
  CellTags->"b:12.0.6"],
Cell[64160, 2119, 423, 8, 79, "Input",
  CellTags->"b:12.0.6"]
}, Closed]],

Cell[CellGroupData[{
Cell[64620, 2132, 173, 5, 53, "Subsection",
  CellTags->{"b:12.0.7", "c:36"}],
Cell[64796, 2139, 476, 10, 148, "Text",
  CellTags->"b:12.0.7"],
Cell[65275, 2151, 67, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[65345, 2155, 1330, 24, 250, "Input",
  CellTags->"b:12.0.7"],
Cell[66678, 2181, 451, 9, 111, "Input",
  CellTags->"b:12.0.7"],
Cell[67132, 2192, 70, 2, 47, "Input",
  CellTags->"b:12.0.7"],
Cell[67205, 2196, 267, 7, 88, "Text",
  CellTags->"b:12.0.7"],
Cell[67475, 2205, 160, 3, 63, "Input"],
Cell[67638, 2210, 45, 1, 47, "Input",
  CellTags->"b:12.0.7"]
}, Closed]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

